@@ -18,19 +18,19 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include <gdk/gdkkeysyms.h>
 
-#include <thunar/thunar-abstract-icon-view.h>
-#include <thunar/thunar-action-manager.h>
-#include <thunar/thunar-gobject-extensions.h>
-#include <thunar/thunar-gtk-extensions.h>
-#include <thunar/thunar-standard-view.h>
-#include <thunar/thunar-preferences.h>
-#include <thunar/thunar-private.h>
-#include <thunar/thunar-window.h>
+#include "thunar/thunar-abstract-icon-view.h"
+#include "thunar/thunar-action-manager.h"
+#include "thunar/thunar-gobject-extensions.h"
+#include "thunar/thunar-gtk-extensions.h"
+#include "thunar/thunar-standard-view.h"
+#include "thunar/thunar-preferences.h"
+#include "thunar/thunar-private.h"
+#include "thunar/thunar-window.h"
 
 
 
@@ -81,6 +81,8 @@ static void         thunar_abstract_icon_view_item_activated          (ExoIconVi
                                                                        ThunarAbstractIconView       *abstract_icon_view);
 static void         thunar_abstract_icon_view_zoom_level_changed      (ThunarAbstractIconView       *abstract_icon_view);
 static void         thunar_abstract_icon_view_queue_redraw            (ThunarStandardView           *standard_view);
+static void         thunar_abstract_icon_view_block_selection_changed (ThunarStandardView *standard_view);
+static void         thunar_abstract_icon_view_unblock_selection_changed (ThunarStandardView *standard_view);
 
 
 
@@ -124,6 +126,8 @@ thunar_abstract_icon_view_class_init (ThunarAbstractIconViewClass *klass)
   thunarstandard_view_class->get_visible_range = thunar_abstract_icon_view_get_visible_range;
   thunarstandard_view_class->highlight_path = thunar_abstract_icon_view_highlight_path;
   thunarstandard_view_class->queue_redraw = thunar_abstract_icon_view_queue_redraw;
+  thunarstandard_view_class->block_selection = thunar_abstract_icon_view_block_selection_changed;
+  thunarstandard_view_class->unblock_selection = thunar_abstract_icon_view_unblock_selection_changed;
 
   /**
    * ThunarAbstractIconView:column-spacing:
@@ -193,6 +197,9 @@ thunar_abstract_icon_view_init (ThunarAbstractIconView *abstract_icon_view)
   gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (view), THUNAR_STANDARD_VIEW (abstract_icon_view)->name_renderer,
                                  "text", THUNAR_COLUMN_NAME);
 
+  /* Set the type of model to be used by the view */
+  g_object_set (G_OBJECT (THUNAR_STANDARD_VIEW (abstract_icon_view)), "model-type", THUNAR_TYPE_LIST_MODEL, NULL);
+
   /* update the icon view on size-allocate events */
   /* TODO: issue not reproducible anymore as of gtk 3.24.18
    * we can probably remove this in the future. */
@@ -541,11 +548,12 @@ thunar_abstract_icon_view_draw (ExoIconView            *view,
   action_entry = thunar_abstract_icon_view_gesture_action (abstract_icon_view, GTK_TEXT_DIR_NONE);
   if (G_LIKELY (action_entry != NULL))
     {
-      gesture_icon = gtk_icon_theme_load_icon (gtk_icon_theme_get_default(),
-                                               action_entry->menu_item_icon_name,
-                                               32 * scale_factor,
-                                               GTK_ICON_LOOKUP_FORCE_SIZE,
-                                               NULL);
+      gesture_icon = gtk_icon_theme_load_icon_for_scale (gtk_icon_theme_get_default(),
+                                                         action_entry->menu_item_icon_name,
+                                                         32,
+                                                         scale_factor,
+                                                         GTK_ICON_LOOKUP_FORCE_SIZE,
+                                                         NULL);
       /* draw the rendered icon */
       if (G_LIKELY (gesture_icon != NULL))
         {
@@ -675,3 +683,25 @@ thunar_abstract_icon_view_queue_redraw (ThunarStandardView *standard_view)
   _thunar_return_if_fail (THUNAR_IS_ABSTRACT_ICON_VIEW (standard_view));
   gtk_widget_queue_draw (gtk_bin_get_child (GTK_BIN (standard_view)));
 }
+
+
+
+static void
+thunar_abstract_icon_view_block_selection_changed (ThunarStandardView *view)
+{
+  _thunar_return_if_fail (THUNAR_IS_STANDARD_VIEW (view));
+
+  g_signal_handlers_block_by_func (G_OBJECT (gtk_bin_get_child (GTK_BIN (view))),
+                                   thunar_standard_view_selection_changed, view);
+}
+
+
+
+static void
+thunar_abstract_icon_view_unblock_selection_changed (ThunarStandardView *view)
+{
+  _thunar_return_if_fail (THUNAR_IS_STANDARD_VIEW (view));
+
+  g_signal_handlers_unblock_by_func (G_OBJECT (gtk_bin_get_child (GTK_BIN (view))),
+                                     thunar_standard_view_selection_changed, view);
+}
