@@ -20,7 +20,7 @@
 #ifndef __THUNAR_ABSTRACT_ICON_VIEW_H__
 #define __THUNAR_ABSTRACT_ICON_VIEW_H__
 
-#include <thunar/thunar-standard-view.h>
+#include "thunar/thunar-standard-view.h"
 
 G_BEGIN_DECLS;
 
