@@ -18,18 +18,19 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include <gdk/gdkkeysyms.h>
 
-#include <thunar/thunar-action-manager.h>
-#include <thunar/thunar-column-editor.h>
-#include <thunar/thunar-details-view.h>
-#include <thunar/thunar-gtk-extensions.h>
-#include <thunar/thunar-private.h>
-#include <thunar/thunar-preferences.h>
-#include <thunar/thunar-window.h>
+#include "thunar/thunar-action-manager.h"
+#include "thunar/thunar-column-editor.h"
+#include "thunar/thunar-details-view.h"
+#include "thunar/thunar-gtk-extensions.h"
+#include "thunar/thunar-private.h"
+#include "thunar/thunar-preferences.h"
+#include "thunar/thunar-window.h"
+#include "thunar/thunar-text-renderer.h"
 
 
 
@@ -38,6 +39,7 @@ enum
 {
   PROP_0,
   PROP_FIXED_COLUMNS,
+  PROP_EXPANDABLE_FOLDERS,
 };
 
 
@@ -92,6 +94,14 @@ static void         thunar_details_view_row_activated           (GtkTreeView
                                                                  GtkTreePath            *path,
                                                                  GtkTreeViewColumn      *column,
                                                                  ThunarDetailsView      *details_view);
+static void         thunar_details_view_row_expanded            (GtkTreeView            *tree_view,
+                                                                 GtkTreeIter            *iter,
+                                                                 GtkTreePath            *path,
+                                                                 ThunarDetailsView      *view);
+static void         thunar_details_view_row_collapsed           (GtkTreeView            *tree_view,
+                                                                 GtkTreeIter            *iter,
+                                                                 GtkTreePath            *path,
+                                                                 ThunarDetailsView      *view);
 static gboolean     thunar_details_view_select_cursor_row       (GtkTreeView            *tree_view,
                                                                  gboolean                editing,
                                                                  ThunarDetailsView      *details_view);
@@ -105,6 +115,8 @@ static void         thunar_details_view_zoom_level_changed      (ThunarDetailsVi
 static gboolean     thunar_details_view_get_fixed_columns       (ThunarDetailsView      *details_view);
 static void         thunar_details_view_set_fixed_columns       (ThunarDetailsView      *details_view,
                                                                  gboolean                fixed_columns);
+static void         thunar_details_view_set_expandable_folders  (ThunarDetailsView      *details_view,
+                                                                 gboolean                expandable_folders);
 static void         thunar_details_view_connect_accelerators    (ThunarStandardView     *standard_view,
                                                                  GtkAccelGroup          *accel_group);
 static void         thunar_details_view_disconnect_accelerators (ThunarStandardView     *standard_view,
@@ -113,7 +125,11 @@ static void         thunar_details_view_append_menu_items       (ThunarStandardV
                                                                  GtkMenu                *menu,
                                                                  GtkAccelGroup          *accel_group);
 static void         thunar_details_view_highlight_option_changed(ThunarDetailsView      *details_view);
-static void         thunar_details_view_draw                    (ThunarDetailsView *details_view);
+static void         thunar_details_view_queue_redraw            (ThunarStandardView     *standard_view);
+static gboolean     thunar_details_view_toggle_expandable_folders (ThunarDetailsView    *details_view);
+static void         thunar_details_view_block_selection_changed   (ThunarStandardView   *standard_view);
+static void         thunar_details_view_unblock_selection_changed (ThunarStandardView   *standard_view);
+
 
 
 struct _ThunarDetailsViewClass
@@ -143,13 +159,18 @@ struct _ThunarDetailsView
   guint idle_id;
 
   GtkCellRenderer   *renderers[THUNAR_N_VISIBLE_COLUMNS];
+
+  ExoTreeView       *tree_view;
+
+  gboolean           expandable_folders;
 };
 
 
 
 static XfceGtkActionEntry thunar_details_view_action_entries[] =
 {
-    { THUNAR_DETAILS_VIEW_ACTION_CONFIGURE_COLUMNS,  "<Actions>/ThunarStandardView/configure-columns", "", XFCE_GTK_MENU_ITEM , N_ ("Configure _Columns..."), N_("Configure the columns in the detailed list view"), NULL, G_CALLBACK (thunar_show_column_editor), },
+    { THUNAR_DETAILS_VIEW_ACTION_CONFIGURE_COLUMNS,           "<Actions>/ThunarStandardView/configure-columns",         "", XFCE_GTK_MENU_ITEM ,       N_ ("Configure _Columns..."),      N_ ("Configure the columns in the detailed list view"), NULL, G_CALLBACK (thunar_show_column_editor),                     },
+    { THUNAR_DETAILS_VIEW_ACTION_TOGGLE_EXPANDABLE_FOLDERS,   "<Actions>/ThunarDetailsView/expandable-folders",         "", XFCE_GTK_CHECK_MENU_ITEM,  N_ ("E_xpandable Folders"),        N_ ("Allow tree like expansion of folders"),            NULL, G_CALLBACK (thunar_details_view_toggle_expandable_folders), },
 };
 
 #define get_action_entry(id) xfce_gtk_get_action_entry_by_id(thunar_details_view_action_entries,G_N_ELEMENTS(thunar_details_view_action_entries),id)
@@ -189,7 +210,10 @@ thunar_details_view_class_init (ThunarDetailsViewClass *klass)
   thunarstandard_view_class->append_menu_items = thunar_details_view_append_menu_items;
   thunarstandard_view_class->connect_accelerators = thunar_details_view_connect_accelerators;
   thunarstandard_view_class->disconnect_accelerators = thunar_details_view_disconnect_accelerators;
+  thunarstandard_view_class->queue_redraw = thunar_details_view_queue_redraw;
   thunarstandard_view_class->zoom_level_property_name = "last-details-view-zoom-level";
+  thunarstandard_view_class->block_selection = thunar_details_view_block_selection_changed;
+  thunarstandard_view_class->unblock_selection = thunar_details_view_unblock_selection_changed;
 
   xfce_gtk_translate_action_entries (thunar_details_view_action_entries, G_N_ELEMENTS (thunar_details_view_action_entries));
 
@@ -205,6 +229,19 @@ thunar_details_view_class_init (ThunarDetailsViewClass *klass)
                                                          "fixed-columns",
                                                          FALSE,
                                                          EXO_PARAM_READWRITE));
+
+  /**
+   * ThunarDetailsView:tree-view:
+   *
+   * %TRUE to enable tree-view.
+   **/
+  g_object_class_install_property (gobject_class,
+                                   PROP_EXPANDABLE_FOLDERS,
+                                   g_param_spec_boolean ("expandable-folders",
+                                                         "expandable-folders",
+                                                         "expandable-folders",
+                                                         FALSE,
+                                                         EXO_PARAM_READWRITE));
 }
 
 
@@ -217,7 +254,6 @@ thunar_details_view_init (ThunarDetailsView *details_view)
   GtkCellRenderer  *left_aligned_renderer;
   GtkCellRenderer  *renderer;
   ThunarColumn      column;
-  GtkWidget        *tree_view;
 
   /* we need to force the GtkTreeView to recalculate column sizes
    * whenever the zoom-level changes, so we connect a handler here.
@@ -225,29 +261,39 @@ thunar_details_view_init (ThunarDetailsView *details_view)
   g_signal_connect (G_OBJECT (details_view), "notify::zoom-level", G_CALLBACK (thunar_details_view_zoom_level_changed), NULL);
 
   /* create the tree view to embed */
-  tree_view = exo_tree_view_new ();
-  g_signal_connect (G_OBJECT (tree_view), "notify::model",
+  details_view->tree_view = EXO_TREE_VIEW (exo_tree_view_new ());
+  g_signal_connect (G_OBJECT (details_view->tree_view), "notify::model",
                     G_CALLBACK (thunar_details_view_notify_model), details_view);
-  g_signal_connect (G_OBJECT (tree_view), "button-press-event",
+  g_signal_connect (G_OBJECT (details_view->tree_view), "button-press-event",
                     G_CALLBACK (thunar_details_view_button_press_event), details_view);
-  g_signal_connect (G_OBJECT (tree_view), "key-press-event",
+  g_signal_connect (G_OBJECT (details_view->tree_view), "key-press-event",
                     G_CALLBACK (thunar_details_view_key_press_event), details_view);
-  g_signal_connect (G_OBJECT (tree_view), "row-activated",
+  g_signal_connect (G_OBJECT (details_view->tree_view), "row-activated",
                     G_CALLBACK (thunar_details_view_row_activated), details_view);
-  g_signal_connect (G_OBJECT (tree_view), "select-cursor-row",
+  g_signal_connect (G_OBJECT (details_view->tree_view), "select-cursor-row",
                     G_CALLBACK (thunar_details_view_select_cursor_row), details_view);
-  gtk_container_add (GTK_CONTAINER (details_view), tree_view);
-  gtk_widget_show (tree_view);
-
-  /* forwards the 'draw' signal down to the tree-view */
-  g_signal_connect_after (G_OBJECT (details_view), "draw",
-                    G_CALLBACK (thunar_details_view_draw), details_view);
+  g_signal_connect (G_OBJECT (details_view->tree_view), "row-expanded",
+                    G_CALLBACK (thunar_details_view_row_expanded), details_view);
+  g_signal_connect (G_OBJECT (details_view->tree_view), "row-collapsed",
+                    G_CALLBACK (thunar_details_view_row_collapsed), details_view);
+  g_signal_connect (G_OBJECT (details_view->tree_view), "select-cursor-row",
+                    G_CALLBACK (thunar_details_view_select_cursor_row), details_view);
+  gtk_container_add (GTK_CONTAINER (details_view), GTK_WIDGET (details_view->tree_view));
+  gtk_widget_show (GTK_WIDGET (details_view->tree_view));
 
   /* configure general aspects of the details view */
-  gtk_tree_view_set_enable_search (GTK_TREE_VIEW (tree_view), TRUE);
+  gtk_tree_view_set_enable_search (GTK_TREE_VIEW (details_view->tree_view), TRUE);
 
   /* enable rubberbanding (if supported) */
-  gtk_tree_view_set_rubber_banding (GTK_TREE_VIEW (tree_view), TRUE);
+  gtk_tree_view_set_rubber_banding (GTK_TREE_VIEW (details_view->tree_view), TRUE);
+
+  /* tell standard view to use tree-model */
+  g_object_set (G_OBJECT (THUNAR_STANDARD_VIEW (details_view)), "model-type", THUNAR_TYPE_TREE_VIEW_MODEL, NULL);
+
+  /* Bind tree-view property between view and preferences object */
+  g_object_bind_property (THUNAR_STANDARD_VIEW (details_view)->preferences, "misc-expandable-folders",
+                          G_OBJECT (details_view), "expandable-folders",
+                          G_BINDING_SYNC_CREATE | G_BINDING_BIDIRECTIONAL);
 
   /* connect to the default column model */
   details_view->column_model = thunar_column_model_get_default ();
@@ -256,26 +302,26 @@ thunar_details_view_init (ThunarDetailsView *details_view)
                           G_CALLBACK (thunar_details_view_row_changed), details_view);
 
   /* allocate the shared right-aligned text renderer */
-  right_aligned_renderer = g_object_new (GTK_TYPE_CELL_RENDERER_TEXT, "xalign", 1.0f, NULL);
+  right_aligned_renderer = g_object_new (thunar_text_renderer_get_type (), "xalign", 1.0f, NULL);
 
   /* this is required in order to disable foreground & background colors on text renderers when the feature is disabled */
   g_object_bind_property (G_OBJECT (THUNAR_STANDARD_VIEW (details_view)->preferences), "misc-highlighting-enabled",
                           G_OBJECT (right_aligned_renderer), "foreground-set",
                           G_BINDING_SYNC_CREATE);
   g_object_bind_property (G_OBJECT (THUNAR_STANDARD_VIEW (details_view)->preferences), "misc-highlighting-enabled",
-                          G_OBJECT (right_aligned_renderer), "background-set",
+                          G_OBJECT (right_aligned_renderer), "highlighting-enabled",
                           G_BINDING_SYNC_CREATE);
   g_object_ref_sink (G_OBJECT (right_aligned_renderer));
 
   /* allocate the shared left-aligned text renderer */
-  left_aligned_renderer = g_object_new (GTK_TYPE_CELL_RENDERER_TEXT, "xalign", 0.0f, NULL);
+  left_aligned_renderer = g_object_new (thunar_text_renderer_get_type (), "xalign", 0.0f, NULL);
 
   /* this is required in order to disable foreground & background colors on text renderers when the feature is disabled */
   g_object_bind_property (G_OBJECT (THUNAR_STANDARD_VIEW (details_view)->preferences), "misc-highlighting-enabled",
                           G_OBJECT (left_aligned_renderer), "foreground-set",
                           G_BINDING_SYNC_CREATE);
   g_object_bind_property (G_OBJECT (THUNAR_STANDARD_VIEW (details_view)->preferences), "misc-highlighting-enabled",
-                          G_OBJECT (left_aligned_renderer), "background-set",
+                          G_OBJECT (left_aligned_renderer), "highlighting-enabled",
                           G_BINDING_SYNC_CREATE);
   g_object_ref_sink (G_OBJECT (left_aligned_renderer));
 
@@ -322,11 +368,11 @@ thunar_details_view_init (ThunarDetailsView *details_view)
         }
 
       /* append the tree view column to the tree view */
-      gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), details_view->columns[column]);
+      gtk_tree_view_append_column (GTK_TREE_VIEW (details_view->tree_view), details_view->columns[column]);
     }
 
   /* configure the tree selection */
-  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree_view));
+  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (details_view->tree_view));
   gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);
   g_signal_connect_swapped (G_OBJECT (selection), "changed",
                             G_CALLBACK (thunar_standard_view_selection_changed), details_view);
@@ -380,6 +426,10 @@ thunar_details_view_get_property (GObject    *object,
       g_value_set_boolean (value, thunar_details_view_get_fixed_columns (details_view));
       break;
 
+    case PROP_EXPANDABLE_FOLDERS:
+      g_value_set_boolean (value,  details_view->expandable_folders);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -402,6 +452,10 @@ thunar_details_view_set_property (GObject      *object,
       thunar_details_view_set_fixed_columns (details_view, g_value_get_boolean (value));
       break;
 
+    case PROP_EXPANDABLE_FOLDERS:
+      thunar_details_view_set_expandable_folders (details_view, g_value_get_boolean (value));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -435,19 +489,6 @@ thunar_details_view_finalize (GObject *object)
 
 
 
-static void
-thunar_details_view_draw (ThunarDetailsView *details_view)
-{
-  GList *children;
-
-  /* redraw as well the corresponding exo_tree_view, which is the only child */
-  children = gtk_container_get_children (GTK_CONTAINER (details_view));
-  for (GList *lp = children; lp != NULL; lp = lp->next)
-    gtk_widget_queue_draw (lp->data);
-}
-
-
-
 static AtkObject*
 thunar_details_view_get_accessible (GtkWidget *widget)
 {
@@ -609,6 +650,10 @@ thunar_details_view_scroll_to_path (ThunarStandardView *standard_view,
 
   _thunar_return_if_fail (THUNAR_IS_DETAILS_VIEW (standard_view));
 
+  /* sometimes StandardView can trigger this before the model has been set in the view */
+  if (gtk_tree_view_get_model (GTK_TREE_VIEW (gtk_bin_get_child (GTK_BIN (standard_view)))) == NULL)
+    return;
+
   /* tell the tree view to scroll to the given row */
   column = gtk_tree_view_get_column (GTK_TREE_VIEW (gtk_bin_get_child (GTK_BIN (standard_view))), 0);
   gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (gtk_bin_get_child (GTK_BIN (standard_view))), path, column, use_align, row_align, col_align);
@@ -725,6 +770,14 @@ thunar_details_view_button_press_event (GtkTreeView       *tree_view,
   GtkTreeViewColumn *column;
   GtkTreeViewColumn *name_column;
   GtkWidget         *window;
+  GtkTreeModel      *model = gtk_tree_view_get_model (tree_view);
+  GtkTreeIter        iter;
+  ThunarFile        *file = NULL;
+  gboolean           row_selected;
+  gint               expander_size, horizontal_separator;
+  gboolean           on_expander = FALSE, activate_on_single_click = FALSE;
+  GdkRectangle       rect;
+  gint               edge_to_expander_region_width;
 
   /* check if the event is for the bin window */
   if (G_UNLIKELY (event->window != gtk_tree_view_get_bin_window (tree_view)))
@@ -756,6 +809,22 @@ thunar_details_view_button_press_event (GtkTreeView       *tree_view,
 
       details_view->button_pressed = TRUE;
 
+      /* find out if the expander was clicked;
+       * only needed if expandable folders is enabled */
+      if (gtk_tree_view_get_show_expanders (tree_view))
+        {
+          gtk_tree_view_get_visible_rect (tree_view, &rect);
+          gtk_widget_style_get (GTK_WIDGET (tree_view),
+                                "expander-size", &expander_size,
+                                "horizontal-separator", &horizontal_separator,
+                                NULL);
+          edge_to_expander_region_width = horizontal_separator / 2 + gtk_tree_path_get_depth (path) * expander_size;
+          if (G_LIKELY (gtk_widget_get_direction (GTK_WIDGET (tree_view)) == GTK_TEXT_DIR_LTR))
+            on_expander = (event->x <= edge_to_expander_region_width);
+          else
+            on_expander = (rect.width - event->x <= edge_to_expander_region_width);
+        }
+
       /* grab the tree view */
       gtk_widget_grab_focus (GTK_WIDGET (tree_view));
 
@@ -764,6 +833,24 @@ thunar_details_view_button_press_event (GtkTreeView       *tree_view,
         {
           gtk_tree_path_free (cursor_path);
 
+          /* if single click is enabled and the click was directed
+           * towards the expander then expand/collapse the row but do not
+           * activate it */
+          g_object_get (tree_view, "single-click", &activate_on_single_click, NULL);
+          if (on_expander && activate_on_single_click)
+            {
+              /* select the path and also set the cursor */
+              gtk_tree_selection_select_path (selection, path);
+              gtk_tree_view_set_cursor (tree_view, path, NULL, FALSE);
+
+              if (gtk_tree_view_row_expanded (tree_view, path))
+                gtk_tree_view_collapse_row (tree_view, path);
+              else
+                gtk_tree_view_expand_row (tree_view, path, FALSE);
+
+              return TRUE;
+            }
+
           if (column != name_column)
             gtk_tree_selection_unselect_all (selection);
 
@@ -774,12 +861,21 @@ thunar_details_view_button_press_event (GtkTreeView       *tree_view,
               /* the clicked row does not have the focus and is not
                * selected, so make it the new selection (start) */
               gtk_tree_selection_unselect_all (selection);
-              gtk_tree_selection_select_path (selection, path);
+              if (gtk_tree_model_get_iter (model, &iter, path))
+                {
+                  file = thunar_standard_view_model_get_file (THUNAR_STANDARD_VIEW_MODEL (model), &iter);
+                  if (file != NULL)
+                    {
+                       row_selected = TRUE;
+                       gtk_tree_selection_select_path (selection, path);
+                       g_object_unref (file);
+                    }
+                }
 
               gtk_tree_path_free (path);
 
-              /* return FALSE to not abort dragging */
-              return FALSE;
+              /* return FALSE to not abort dragging; when row was selected */
+              return !row_selected;
             }
           gtk_tree_path_free (path);
         }
@@ -810,7 +906,15 @@ thunar_details_view_button_press_event (GtkTreeView       *tree_view,
               if (!gtk_tree_selection_path_is_selected (selection, path))
                 {
                   gtk_tree_selection_unselect_all (selection);
-                  gtk_tree_selection_select_path (selection, path);
+                  if (gtk_tree_model_get_iter (model, &iter, path))
+                    {
+                      file = thunar_standard_view_model_get_file (THUNAR_STANDARD_VIEW_MODEL (model), &iter);
+                      if (file != NULL)
+                        {
+                          g_object_unref (file);
+                          gtk_tree_selection_select_path (selection, path);
+                        }
+                    }
                 }
 
               /* queue the menu popup */
@@ -828,10 +932,17 @@ thunar_details_view_button_press_event (GtkTreeView       *tree_view,
         {
           /* select only the path to the item on which the user clicked */
           gtk_tree_selection_unselect_all (selection);
-          gtk_tree_selection_select_path (selection, path);
-
-          /* try to open the path as new window/tab, if possible */
-          _thunar_standard_view_open_on_middle_click (THUNAR_STANDARD_VIEW (details_view), path, event->state);
+          if (gtk_tree_model_get_iter (model, &iter, path))
+            {
+              file = thunar_standard_view_model_get_file (THUNAR_STANDARD_VIEW_MODEL (model), &iter);
+              if (file != NULL)
+                {
+                  g_object_unref (file);
+                  gtk_tree_selection_select_path (selection, path);
+                  /* try to open the path as new window/tab, if possible */
+                  _thunar_standard_view_open_on_middle_click (THUNAR_STANDARD_VIEW (details_view), path, event->state);
+                }
+            }
 
           /* cleanup */
           gtk_tree_path_free (path);
@@ -845,11 +956,149 @@ thunar_details_view_button_press_event (GtkTreeView       *tree_view,
 
 
 
+static gboolean
+tree_view_set_cursor_if_file_not_null (GtkTreeView  *tree_view,
+                                       GtkTreeModel *model,
+                                       GtkTreePath  *path)
+{
+  ThunarFile  *file;
+  GtkTreeIter  iter;
+
+  /* sets the cursor if file != NULL for the iter;
+   * if cursor set then TRUE is returned; FALSE otherwise */
+
+  _thunar_return_val_if_fail (GTK_IS_TREE_VIEW (tree_view), FALSE);
+  _thunar_return_val_if_fail (GTK_IS_TREE_MODEL (model), FALSE);
+  _thunar_return_val_if_fail (path != NULL, FALSE);
+
+  if (!gtk_tree_model_get_iter (model, &iter, path))
+    return FALSE;
+
+  file = thunar_standard_view_model_get_file (THUNAR_STANDARD_VIEW_MODEL (model), &iter);
+
+  if (file == NULL)
+    return FALSE;
+
+  g_object_unref (file);
+  gtk_tree_view_set_cursor (tree_view, path, NULL, FALSE);
+  return TRUE;
+}
+
+
+
+static void
+thunar_details_view_key_up_set_cursor (GtkTreeView  *tree_view,
+                                       GtkTreeModel *model,
+                                       GtkTreePath  *cursor)
+{
+  GtkTreeIter  iter;
+  gint        *indices;
+  gint         depth;
+
+  _thunar_return_if_fail (GTK_IS_TREE_VIEW (tree_view));
+  _thunar_return_if_fail (GTK_IS_TREE_MODEL (model));
+  _thunar_return_if_fail (cursor != NULL);
+
+  if (!gtk_tree_path_prev (cursor))
+    {
+      /* if we are at the first most toplevel node we have nothing do */
+      if (gtk_tree_path_get_depth (cursor) <= 1)
+        return;
+
+      /* it might be the first child of an expanded node;
+       * in this case we want to point to the parent */
+      gtk_tree_path_up (cursor);
+
+      /* the parent path we got now should be perfectly fine;
+       * but just in case setting the cursor fails
+       * we go and set the prev node of the parent */
+      if (G_UNLIKELY (!tree_view_set_cursor_if_file_not_null (tree_view, model, cursor)))
+        thunar_details_view_key_up_set_cursor(tree_view, model, cursor);
+
+      return;
+    }
+
+  /* set cursor if no child visible */
+  if (!gtk_tree_view_row_expanded (tree_view, cursor))
+    {
+      /* If we are at a row that is loading set cursor will fail;
+       * try the previous node */
+      if (G_UNLIKELY (!tree_view_set_cursor_if_file_not_null (tree_view, model, cursor)))
+        thunar_details_view_key_up_set_cursor (tree_view, model, cursor);
+      return;
+    }
+
+  /* get the iter for the cursor */
+  if (G_UNLIKELY (!gtk_tree_model_get_iter (model, &iter, cursor)))
+    return;
+
+  /* set cursor to the last visible child;
+   * Works recursively to point to the truly last child */
+  gtk_tree_path_down (cursor);
+  indices = gtk_tree_path_get_indices_with_depth (cursor, &depth);
+
+  /* point to the last child */
+  if (depth > 0)
+    indices[depth - 1] = gtk_tree_model_iter_n_children (model, &iter);
+
+  /* If we are at a row that is loading set cursor will fail;
+   * try the previous node */
+  if (G_UNLIKELY (!tree_view_set_cursor_if_file_not_null (tree_view, model, cursor)))
+      thunar_details_view_key_up_set_cursor (tree_view, model, cursor);
+}
+
+
+
+static void
+thunar_details_view_key_down_set_cursor (GtkTreeView  *tree_view,
+                                         GtkTreeModel *model,
+                                         GtkTreePath  *cursor)
+{
+  _thunar_return_if_fail (GTK_IS_TREE_VIEW (tree_view));
+  _thunar_return_if_fail (GTK_IS_TREE_MODEL (model));
+  _thunar_return_if_fail (cursor != NULL);
+
+  if (!gtk_tree_view_row_expanded (tree_view, cursor))
+    {
+      gtk_tree_path_next (cursor);
+
+      if (tree_view_set_cursor_if_file_not_null (tree_view, model, cursor)
+          || gtk_tree_path_get_depth(cursor) <= 1)
+        return;
+
+      if (!gtk_tree_path_up (cursor))
+        return;
+
+      gtk_tree_path_next (cursor);
+
+      /* If we are at a row that is loading set cursor will fail;
+       * try the next node */
+      if (G_UNLIKELY (!tree_view_set_cursor_if_file_not_null (tree_view, model, cursor)))
+        thunar_details_view_key_down_set_cursor (tree_view, model, cursor);
+
+      return;
+    }
+  gtk_tree_path_down (cursor);
+
+  /* If we are at a row that is loading set cursor will fail;
+   * try the next node */
+  if (G_UNLIKELY (!tree_view_set_cursor_if_file_not_null (tree_view, model, cursor)))
+    thunar_details_view_key_down_set_cursor (tree_view, model, cursor);
+}
+
+
+
 static gboolean
 thunar_details_view_key_press_event (GtkTreeView       *tree_view,
                                      GdkEventKey       *event,
                                      ThunarDetailsView *details_view)
 {
+  GtkTreePath      *path;
+  gboolean          stopPropagation = FALSE;
+  GtkTreeModel     *model = gtk_tree_view_get_model (tree_view);
+  GtkTreeIter       iter;
+  ThunarFile       *file = NULL;
+
   details_view->button_pressed = FALSE;
 
   /* popup context menu if "Menu" or "<Shift>F10" is pressed */
@@ -859,7 +1108,83 @@ thunar_details_view_key_press_event (GtkTreeView       *tree_view,
       return TRUE;
     }
 
-  return FALSE;
+  /* don't allow keyboard nav for tree-view if tree-view isn't allowed */
+  if (details_view->expandable_folders == FALSE)
+    return FALSE;
+
+  /* Get path of currently highlighted item */
+  gtk_tree_view_get_cursor(tree_view, &path, NULL);
+
+  if (path == NULL)
+    return stopPropagation;
+
+  switch (event->keyval)
+    {
+    case GDK_KEY_Up:
+    case GDK_KEY_KP_Up:
+      thunar_details_view_key_up_set_cursor (tree_view, model, path);
+      stopPropagation = TRUE;
+      break;
+
+    case GDK_KEY_Down:
+    case GDK_KEY_KP_Down:
+      thunar_details_view_key_down_set_cursor (tree_view, model, path);
+      stopPropagation = TRUE;
+      break;
+
+    case GDK_KEY_Left:
+    case GDK_KEY_KP_Left:
+      /* if branch is expanded then collapse it */
+      if (gtk_tree_view_row_expanded (tree_view, path))
+        {
+          gtk_tree_view_collapse_row (tree_view, path);
+        }
+      else /* if the branch is already collapsed */
+        {
+          if (gtk_tree_path_get_depth (path) > 1 && gtk_tree_path_up (path))
+            {
+              /* if this is not a toplevel item then move to parent */
+              gtk_tree_view_set_cursor (tree_view, path, NULL, FALSE);
+            }
+        }
+
+      stopPropagation = TRUE;
+      break;
+
+    case GDK_KEY_Right:
+    case GDK_KEY_KP_Right:
+      /* if branch is not expanded then expand it */
+      if (!gtk_tree_view_row_expanded (tree_view, path))
+        {
+          gtk_tree_view_expand_row (tree_view, path, FALSE);
+        }
+      else /* if branch is already expanded then move to first child */
+        {
+          if (gtk_tree_model_get_iter (model, &iter, path))
+            {
+              gtk_tree_model_get (model, &iter,
+                                  THUNAR_COLUMN_FILE, &file, -1);
+              if (file != NULL)
+                gtk_tree_view_set_cursor (tree_view, path, NULL, FALSE);
+            }
+        }
+      stopPropagation = TRUE;
+      break;
+
+    case GDK_KEY_space:
+    case GDK_KEY_Return:
+    case GDK_KEY_KP_Enter:
+      /* Allow default actions to handle this case */
+      GTK_WIDGET_CLASS (thunar_details_view_parent_class)->key_press_event (GTK_WIDGET (tree_view), event);
+      stopPropagation = TRUE;
+      break;
+    }
+
+  gtk_tree_path_free (path);
+  if (stopPropagation)
+    gtk_widget_grab_focus (GTK_WIDGET (tree_view));
+
+  return stopPropagation;
 }
 
 
@@ -891,6 +1216,33 @@ thunar_details_view_row_activated (GtkTreeView       *tree_view,
 
 
 
+static void
+thunar_details_view_row_expanded (GtkTreeView       *tree_view,
+                                  GtkTreeIter       *parent,
+                                  GtkTreePath       *path,
+                                  ThunarDetailsView *view)
+{
+  GtkTreeModel *model = gtk_tree_view_get_model (tree_view);
+
+  /* this will replace the dummy child with actual children */
+  thunar_tree_view_model_load_subdir (THUNAR_TREE_VIEW_MODEL (model), parent);
+}
+
+
+
+static void
+thunar_details_view_row_collapsed (GtkTreeView       *tree_view,
+                                   GtkTreeIter       *iter,
+                                   GtkTreePath       *path,
+                                   ThunarDetailsView *view)
+{
+  /* schedule a cleanup with a delay. If the row is expanded before
+   * the scheduled cleanup then the cleanup is cancelled. */
+  thunar_tree_view_model_schedule_unload (THUNAR_TREE_VIEW_MODEL (THUNAR_STANDARD_VIEW (view)->model), iter);
+}
+
+
+
 static gboolean
 thunar_details_view_select_cursor_row (GtkTreeView            *tree_view,
                                        gboolean                editing,
@@ -1069,6 +1421,7 @@ thunar_details_view_append_menu_items (ThunarStandardView *standard_view,
   _thunar_return_if_fail (THUNAR_IS_DETAILS_VIEW (details_view));
 
   xfce_gtk_menu_item_new_from_action_entry (get_action_entry (THUNAR_DETAILS_VIEW_ACTION_CONFIGURE_COLUMNS), G_OBJECT (details_view), GTK_MENU_SHELL (menu));
+  xfce_gtk_toggle_menu_item_new_from_action_entry (get_action_entry (THUNAR_DETAILS_VIEW_ACTION_TOGGLE_EXPANDABLE_FOLDERS), G_OBJECT (details_view), details_view->expandable_folders, GTK_MENU_SHELL (menu));
 }
 
 
@@ -1162,6 +1515,34 @@ thunar_details_view_set_fixed_columns (ThunarDetailsView *details_view,
 
 
 
+/**
+ * thunar_details_view_set_expandable_folders:
+ * @details_view  : a #ThunarDetailsView.
+ * @expandable_folders : %TRUE to enable expansion of folders.
+ *
+ * Sets @expandable_folders in @details_view.
+ * Accordingly, updates view to show expanders and tree-lines.
+ **/
+static void
+thunar_details_view_set_expandable_folders (ThunarDetailsView *details_view,
+                                            gboolean           expandable_folders)
+{
+  _thunar_return_if_fail (THUNAR_IS_DETAILS_VIEW (details_view));
+
+  details_view->expandable_folders = expandable_folders;
+
+  gtk_tree_view_set_show_expanders (GTK_TREE_VIEW (details_view->tree_view), details_view->expandable_folders);
+  gtk_tree_view_set_enable_tree_lines (GTK_TREE_VIEW (details_view->tree_view), details_view->expandable_folders);
+
+  if (details_view->expandable_folders == FALSE)
+    gtk_tree_view_collapse_all (GTK_TREE_VIEW (details_view->tree_view));
+
+  /* notify listeners */
+  g_object_notify (G_OBJECT (details_view), "expandable-folders");
+}
+
+
+
 /**
  * thunar_details_view_set_date_deleted_column_visible:
  * @details_view  : a #ThunarDetailsView.
@@ -1215,6 +1596,64 @@ thunar_details_view_highlight_option_changed (ThunarDetailsView *details_view)
         continue;
       gtk_tree_view_column_set_cell_data_func (GTK_TREE_VIEW_COLUMN (details_view->columns[column]),
                                                GTK_CELL_RENDERER (details_view->renderers[column]),
-                                               function, NULL, NULL);
+                                               function, THUNAR_STANDARD_VIEW (details_view), NULL);
     }
 }
+
+
+
+static void
+thunar_details_view_queue_redraw (ThunarStandardView *standard_view)
+{
+  ThunarDetailsView *details_view = THUNAR_DETAILS_VIEW (standard_view);
+
+  _thunar_return_if_fail (THUNAR_IS_DETAILS_VIEW (details_view));
+
+  gtk_widget_queue_draw (GTK_WIDGET (details_view->tree_view));
+}
+
+
+
+/**
+ * thunar_details_view_toggle_expandable_folders:
+ * @details_view  : a #ThunarDetailsView.
+ *
+ * Toggles the expandable folders feature.
+ **/
+static gboolean
+thunar_details_view_toggle_expandable_folders (ThunarDetailsView *details_view)
+{
+  _thunar_return_val_if_fail (THUNAR_IS_DETAILS_VIEW (details_view), FALSE);
+
+  thunar_details_view_set_expandable_folders (details_view, !details_view->expandable_folders);
+
+  return TRUE;
+}
+
+
+
+static void
+thunar_details_view_block_selection_changed (ThunarStandardView *view)
+{
+  ThunarDetailsView *details_view = THUNAR_DETAILS_VIEW (view);
+  GtkTreeSelection  *selection;
+
+  _thunar_return_if_fail (THUNAR_IS_DETAILS_VIEW (details_view));
+
+  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (details_view->tree_view));
+  g_signal_handlers_block_by_func (G_OBJECT (selection), thunar_standard_view_selection_changed, view);
+}
+
+
+
+static void
+thunar_details_view_unblock_selection_changed (ThunarStandardView *view)
+{
+  ThunarDetailsView *details_view = THUNAR_DETAILS_VIEW (view);
+  GtkTreeSelection  *selection;
+
+  _thunar_return_if_fail (THUNAR_IS_DETAILS_VIEW (details_view));
+
+  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (details_view->tree_view));
+  g_signal_handlers_unblock_by_func (G_OBJECT (selection), thunar_standard_view_selection_changed, view);
+}
