@@ -263,8 +263,6 @@ void              thunar_file_set_file_count             (ThunarFile
                                                           const guint             count);
 
 GList            *thunar_file_get_emblem_names           (ThunarFile              *file);
-void              thunar_file_set_emblem_names           (ThunarFile              *file,
-                                                          GList                   *emblem_names);
 
 const gchar      *thunar_file_get_custom_icon            (const ThunarFile        *file);
 gboolean          thunar_file_set_custom_icon            (ThunarFile              *file,
@@ -275,9 +273,6 @@ const gchar         *thunar_file_get_thumbnail_path      (ThunarFile
                                                           ThunarThumbnailSize      thumbnail_size);
 ThunarFileThumbState thunar_file_get_thumb_state         (const ThunarFile        *file,
                                                           ThunarThumbnailSize      size);
-void                 thunar_file_set_thumb_state         (ThunarFile              *file,
-                                                          ThunarFileThumbState     state,
-                                                          ThunarThumbnailSize      size);
 GIcon            *thunar_file_get_preview_icon           (const ThunarFile        *file);
 const gchar      *thunar_file_get_icon_name              (ThunarFile              *file,
                                                           ThunarFileIconState      icon_state,
@@ -308,7 +303,7 @@ GList            *thunar_file_list_to_thunar_g_file_list (GList
 
 gboolean          thunar_file_is_desktop                 (const ThunarFile *file);
 
-const gchar*      thunar_file_get_metadata_setting       (ThunarFile             *file,
+gchar            *thunar_file_get_metadata_setting       (ThunarFile             *file,
                                                           const gchar            *setting_name);
 void              thunar_file_set_metadata_setting       (ThunarFile             *file,
                                                           const gchar            *setting_name,
@@ -370,6 +365,11 @@ gint              thunar_cmp_files_by_size_and_items_count      (ThunarFile
 gint              thunar_cmp_files_by_type                      (const ThunarFile   *a,
                                                                  const ThunarFile   *b,
                                                                  gboolean            case_sensitive);
+void              thunar_file_request_thumbnail                 (ThunarFile         *file,
+                                                                 ThunarThumbnailSize size);
+void              thunar_file_update_thumbnail                  (ThunarFile          *file,
+                                                                 ThunarFileThumbState state,
+                                                                 ThunarThumbnailSize  size);
 
 /**
  * thunar_file_is_root:
