@@ -25,6 +25,14 @@
 
 G_BEGIN_DECLS
 
+#define THUNAR_METADATA_STRING_DELIMETER "|"
+
+typedef enum
+{
+  THUNAR_GTYPE_STRING,
+  THUNAR_GTYPE_STRINGV
+} ThunarGType;
+
 GFile       *thunar_g_file_new_for_home             (void);
 GFile       *thunar_g_file_new_for_root             (void);
 GFile       *thunar_g_file_new_for_recent           (void);
@@ -34,7 +42,7 @@ GFile       *thunar_g_file_new_for_computer         (void);
 GFile       *thunar_g_file_new_for_network          (void);
 GFile       *thunar_g_file_new_for_bookmarks        (void);
 
-GFile       *thunar_g_file_new_for_symlink_target   (GFile                *file);
+GFile       *thunar_g_file_resolve_symlink          (GFile                *file);
 
 gboolean     thunar_g_file_is_root                  (GFile                *file);
 gboolean     thunar_g_file_is_trashed               (GFile                *file);
@@ -124,17 +132,18 @@ gboolean     thunar_g_file_is_in_xdg_data_dir          (GFile             *file)
 gboolean     thunar_g_file_is_desktop_file             (GFile             *file);
 char        *thunar_g_file_get_link_path_for_symlink   (GFile             *file_to_link,
                                                         GFile             *symlink);
-char        *thunar_g_file_get_resolved_path           (GFile             *file);
 void         thunar_g_file_set_metadata_setting        (GFile             *file,
                                                         GFileInfo         *info,
+                                                        ThunarGType        type,
                                                         const gchar       *setting_name,
                                                         const gchar       *setting_value,
                                                         gboolean           async);
 void         thunar_g_file_clear_metadata_setting      (GFile             *file,
                                                         GFileInfo         *info,
                                                         const gchar       *setting_name);
-const gchar *thunar_g_file_get_metadata_setting        (GFile             *file,
+ gchar      *thunar_g_file_get_metadata_setting        (GFile             *file,
                                                         GFileInfo         *info,
+                                                        ThunarGType        type,
                                                         const gchar       *setting_name);
 char        *thunar_g_file_get_content_type            (GFile             *file);
 G_END_DECLS
