@@ -35,7 +35,6 @@
 #include "thunar/thunar-preferences.h"
 #include "thunar/thunar-private.h"
 #include "thunar/thunar-util.h"
-#include "thunar/thunar-thumbnailer.h"
 
 
 
@@ -126,8 +125,6 @@ struct _ThunarIconFactory
 
   /* stamp that gets bumped when the theme changes */
   guint                theme_stamp;
-
-  ThunarThumbnailer   *thumbnailer;
 };
 
 struct _ThunarIconKey
@@ -251,8 +248,6 @@ thunar_icon_factory_init (ThunarIconFactory *factory)
   factory->thumbnail_mode = THUNAR_THUMBNAIL_MODE_ONLY_LOCAL;
   factory->thumbnail_size = THUNAR_THUMBNAIL_SIZE_NORMAL;
 
-  factory->thumbnailer = thunar_thumbnailer_get ();
-
   /* connect emission hook for the "changed" signal on the GtkIconTheme class. We use the emission
    * hook way here, because that way we can make sure that the icon cache is definetly cleared
    * before any other part of the application gets notified about the icon theme change.
@@ -921,7 +916,6 @@ thunar_icon_factory_load_file_icon (ThunarIconFactory  *factory,
   const gchar     *icon_name;
   const gchar     *custom_icon;
   ThunarIconStore *store;
-  guint            request;
 
   _thunar_return_val_if_fail (THUNAR_IS_ICON_FACTORY (factory), NULL);
   _thunar_return_val_if_fail (THUNAR_IS_FILE (file), NULL);
@@ -1012,9 +1006,6 @@ thunar_icon_factory_load_file_icon (ThunarIconFactory  *factory,
               if (thumbnail_path != NULL)
                 /* try to load the thumbnail */
                 icon = thunar_icon_factory_load_from_file (factory, thumbnail_path, icon_size, scale_factor);
-              else if (thunar_file_get_thumb_state (file, thunar_icon_size_to_thumbnail_size (icon_size * scale_factor)) != THUNAR_FILE_THUMB_STATE_NONE)
-                /* thumbnail does not exist; so load it */
-                thunar_thumbnailer_queue_file (factory->thumbnailer, file, &request, thunar_icon_size_to_thumbnail_size (icon_size * scale_factor));
             }
         }
     }
