@@ -19,17 +19,17 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include <glib.h>
 #include <glib-object.h>
 
-#include <thunar/thunar-application.h>
-#include <thunar/thunar-file-monitor.h>
-#include <thunar/thunar-image.h>
-#include <thunar/thunar-icon-factory.h>
-#include <thunar/thunar-private.h>
+#include "thunar/thunar-application.h"
+#include "thunar/thunar-file-monitor.h"
+#include "thunar/thunar-image.h"
+#include "thunar/thunar-icon-factory.h"
+#include "thunar/thunar-private.h"
 
 
 
@@ -208,7 +208,7 @@ thunar_image_update (ThunarImage *image)
 
       scale_factor = gtk_widget_get_scale_factor (GTK_WIDGET (image));
       icon = thunar_icon_factory_load_file_icon (icon_factory, image->priv->file,
-                                                 THUNAR_FILE_ICON_STATE_DEFAULT, 48 * scale_factor);
+                                                 THUNAR_FILE_ICON_STATE_DEFAULT, 48, scale_factor);
 
       surface = gdk_cairo_surface_create_from_pixbuf (icon, scale_factor, gtk_widget_get_window (GTK_WIDGET (image)));
       gtk_image_set_from_surface (GTK_IMAGE (image), surface);
