@@ -16,12 +16,12 @@
  * this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <thunar/thunar-dialogs.h>
-#include <thunar/thunar-enum-types.h>
-#include <thunar/thunar-job-operation-history.h>
-#include <thunar/thunar-notify.h>
-#include <thunar/thunar-preferences.h>
-#include <thunar/thunar-private.h>
+#include "thunar/thunar-dialogs.h"
+#include "thunar/thunar-enum-types.h"
+#include "thunar/thunar-job-operation-history.h"
+#include "thunar/thunar-notify.h"
+#include "thunar/thunar-preferences.h"
+#include "thunar/thunar-private.h"
 #include <libxfce4ui/libxfce4ui.h>
 
 /**
@@ -307,6 +307,7 @@ thunar_job_operation_history_undo (void)
   gchar              *file_uri;
   GError             *err = NULL;
   const GList        *overwritten_files;
+  gboolean            operation_canceled;
 
   g_mutex_lock (&job_operation_history->job_operation_list_mutex);
 
@@ -366,12 +367,21 @@ thunar_job_operation_history_undo (void)
       }
 
     inverted_operation = thunar_job_operation_new_invert (operation_marker);
-    thunar_job_operation_execute (inverted_operation, &err);
+    operation_canceled = thunar_job_operation_execute (inverted_operation, &err);
     g_object_unref (inverted_operation);
 
-    if (err == NULL)
+    if (err == NULL && !operation_canceled)
       thunar_notify_undo (operation_marker);
 
+    if (err != NULL)
+      {
+        xfce_dialog_show_warning (NULL,
+                                  err->message,
+                                  _("Failed to undo operation '%s'"),
+                                  thunar_job_operation_get_kind_nick (operation_marker));
+        g_clear_error (&err);
+      }
+
     g_mutex_unlock (&job_operation_history->job_operation_list_mutex);
 
     /* Notify all subscribers of our properties */
@@ -394,6 +404,7 @@ thunar_job_operation_history_redo (void)
   gchar              *file_uri;
   GError             *err = NULL;
   const GList        *overwritten_files;
+  gboolean            operation_canceled;
 
   g_mutex_lock (&job_operation_history->job_operation_list_mutex);
 
@@ -451,10 +462,19 @@ thunar_job_operation_history_redo (void)
         g_string_free (warning_body, TRUE);
       }
 
-    thunar_job_operation_execute (operation_marker, &err);
+    operation_canceled = thunar_job_operation_execute (operation_marker, &err);
 
-    if (err == NULL)
-      thunar_notify_redo (operation_marker);
+    if (err == NULL && !operation_canceled)
+      thunar_notify_undo (operation_marker);
+
+    if (err != NULL)
+      {
+        xfce_dialog_show_warning (NULL,
+                                  err->message,
+                                  _("Failed to redo operation '%s'"),
+                                  thunar_job_operation_get_kind_nick (operation_marker));
+        g_clear_error (&err);
+      }
 
     g_mutex_unlock (&job_operation_history->job_operation_list_mutex);
 
@@ -521,3 +541,52 @@ thunar_job_operation_history_can_redo (void)
   return TRUE;
 }
 
+
+
+/**
+ * thunar_job_operation_history_get_undo_text:
+ *
+ * Returns the description of the undo action
+ * The returned string should be freed with g_free() when no longer needed.
+ *
+ * Return value: a newly-allocated string holding the text
+ **/
+gchar*
+thunar_job_operation_history_get_undo_text (void)
+{
+  if (thunar_job_operation_history_can_undo ())
+    {
+      gchar *action_text = thunar_job_operation_get_action_text (job_operation_history->lp_undo->data);
+      /* TRANSLATORS: An example: 'Undo the latest 'copy' operation (2 files) */
+      gchar *final_text = g_strdup_printf (_("Undo the latest '%s' operation (%s)"),  thunar_job_operation_get_kind_nick (job_operation_history->lp_undo->data), action_text);
+      g_free (action_text);
+      return final_text;
+    }
+  
+  return g_strdup (gettext ("Undo the latest operation"));
+}
+
+
+
+/**
+ * thunar_job_operation_history_get_redo_text
+ * 
+ * Returns the description of the redo action. 
+ * The returned string should be freed with g_free() when no longer needed.
+ *
+ * Return value: a newly-allocated string holding the text
+ **/
+gchar*
+thunar_job_operation_history_get_redo_text (void)
+{
+  if (thunar_job_operation_history_can_redo ())
+    {
+      gchar *action_text = thunar_job_operation_get_action_text (job_operation_history->lp_redo->data);
+      /* TRANSLATORS: An example: 'Redo the latest 'copy' operation (2 files) */
+      gchar *final_text = g_strdup_printf (_("Redo the latest '%s' operation (%s)"),  thunar_job_operation_get_kind_nick (job_operation_history->lp_redo->data), action_text);
+      g_free (action_text);
+      return final_text;
+    }
+  
+  return g_strdup (gettext ("Redo the latest operation"));
+}
