@@ -20,7 +20,7 @@
 #define __THUNAR_JOB_OPERATION_H__
 
 #include <glib.h>
-#include <thunar/thunar-enum-types.h>
+#include "thunar/thunar-enum-types.h"
 
 G_BEGIN_DECLS
 
@@ -49,7 +49,7 @@ const gchar            *thunar_job_operation_get_kind_nick         (ThunarJobOpe
 ThunarJobOperationKind  thunar_job_operation_get_kind              (ThunarJobOperation    *job_operation);
 const GList            *thunar_job_operation_get_overwritten_files (ThunarJobOperation    *job_operation);
 gboolean                thunar_job_operation_empty                 (ThunarJobOperation    *job_operation);
-
+gchar                  *thunar_job_operation_get_action_text       (ThunarJobOperation    *job_operation);
 
 G_END_DECLS
 
