@@ -23,7 +23,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #ifdef HAVE_ERRNO_H
@@ -38,14 +38,14 @@
 
 #include <gdk/gdkkeysyms.h>
 
-#include <thunar/thunar-gobject-extensions.h>
-#include <thunar/thunar-icon-factory.h>
-#include <thunar/thunar-icon-renderer.h>
-#include <thunar/thunar-list-model.h>
-#include <thunar/thunar-path-entry.h>
-#include <thunar/thunar-private.h>
-#include <thunar/thunar-util.h>
-#include <thunar/thunar-window.h>
+#include "thunar/thunar-gobject-extensions.h"
+#include "thunar/thunar-icon-factory.h"
+#include "thunar/thunar-icon-renderer.h"
+#include "thunar/thunar-list-model.h"
+#include "thunar/thunar-path-entry.h"
+#include "thunar/thunar-private.h"
+#include "thunar/thunar-util.h"
+#include "thunar/thunar-window.h"
 
 
 
@@ -231,7 +231,7 @@ thunar_path_entry_init (ThunarPathEntry *path_entry)
 {
   GtkEntryCompletion *completion;
   GtkCellRenderer    *renderer;
-  ThunarListModel    *store;
+  ThunarStandardViewModel *store;
 
   path_entry->check_completion_idle_id = 0;
   path_entry->working_directory = NULL;
@@ -254,8 +254,9 @@ thunar_path_entry_init (ThunarPathEntry *path_entry)
 
   /* allocate a new list mode for the completion */
   store = thunar_list_model_new ();
-  thunar_list_model_set_show_hidden (store, TRUE);
-  thunar_list_model_set_folders_first (store, TRUE);
+  thunar_list_model_check_file_in_model_before_use (THUNAR_LIST_MODEL (store));
+  thunar_standard_view_model_set_show_hidden (store, TRUE);
+  thunar_standard_view_model_set_folders_first (store, TRUE);
   gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (store), THUNAR_COLUMN_FILE_NAME, GTK_SORT_ASCENDING);
   gtk_entry_completion_set_model (completion, GTK_TREE_MODEL (store));
   g_object_unref (G_OBJECT (store));
@@ -470,7 +471,7 @@ thunar_path_entry_motion_notify_event (GtkWidget      *widget,
       icon = thunar_icon_factory_load_file_icon (path_entry->icon_factory,
                                                  path_entry->current_file,
                                                  THUNAR_FILE_ICON_STATE_DEFAULT,
-                                                 MAX (size, 16) * scale_factor);
+                                                 MAX (size, 16), scale_factor);
       if (G_LIKELY (icon != NULL))
         {
           surface = gdk_cairo_surface_create_from_pixbuf (icon, scale_factor, gtk_widget_get_window (widget));
@@ -670,7 +671,7 @@ thunar_path_entry_changed (GtkEditable *editable)
       model = gtk_entry_completion_get_model (completion);
       g_object_ref (G_OBJECT (model));
       gtk_entry_completion_set_model (completion, NULL);
-      thunar_list_model_set_folder (THUNAR_LIST_MODEL (model), folder, NULL);
+      thunar_standard_view_model_set_folder (THUNAR_STANDARD_VIEW_MODEL (model), folder, NULL);
       gtk_entry_completion_set_model (completion, model);
       g_object_unref (G_OBJECT (model));
 
@@ -747,14 +748,14 @@ thunar_path_entry_update_icon (ThunarPathEntry *path_entry)
       icon = thunar_icon_factory_load_file_icon (path_entry->icon_factory,
                                                  path_entry->current_file,
                                                  THUNAR_FILE_ICON_STATE_DEFAULT,
-                                                 icon_size * scale_factor);
+                                                 icon_size, scale_factor);
     }
   else if (G_LIKELY (path_entry->current_folder != NULL))
     {
       icon = thunar_icon_factory_load_file_icon (path_entry->icon_factory,
                                                  path_entry->current_folder,
                                                  THUNAR_FILE_ICON_STATE_DEFAULT,
-                                                 icon_size * scale_factor);
+                                                 icon_size, scale_factor);
     }
 
   if (icon != NULL)
@@ -1188,8 +1189,6 @@ thunar_path_entry_check_completion_idle (gpointer user_data)
   ThunarPathEntry *path_entry = THUNAR_PATH_ENTRY (user_data);
   const gchar     *text;
 
-THUNAR_THREADS_ENTER
-
   /* check if the user entered at least part of a filename */
   text = gtk_entry_get_text (GTK_ENTRY (path_entry));
   if (*text != '\0' && text[strlen (text) - 1] != '/')
@@ -1198,8 +1197,6 @@ THUNAR_THREADS_ENTER
       thunar_path_entry_common_prefix_append (path_entry, TRUE);
     }
 
-THUNAR_THREADS_LEAVE
-
   return FALSE;
 }
 
@@ -1372,10 +1369,10 @@ thunar_path_entry_get_search_query (ThunarPathEntry *path_entry)
 {
   unsigned long search_prefix_length;
   const gchar *text = gtk_entry_get_text (GTK_ENTRY (path_entry));
-  search_prefix_length = strlen (SEARCH_PREFIX);
+  search_prefix_length = strlen (thunar_util_get_search_prefix ());
 
   _thunar_return_val_if_fail (THUNAR_IS_PATH_ENTRY (path_entry), NULL);
-  _thunar_return_val_if_fail (strncmp (text, SEARCH_PREFIX, search_prefix_length) == 0, NULL);
+  _thunar_return_val_if_fail (strncmp (text, thunar_util_get_search_prefix (), search_prefix_length) == 0, NULL);
 
   return strlen(text) > search_prefix_length ? g_strdup (&text[search_prefix_length]) : g_strdup ("");
 }
