@@ -18,7 +18,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #ifdef HAVE_MEMORY_H
@@ -30,19 +30,21 @@
 
 #include <libxfce4ui/libxfce4ui.h>
 
-#include <thunar/thunar-abstract-dialog.h>
-#include <thunar/thunar-action-manager.h>
-#include <thunar/thunar-application.h>
-#include <thunar/thunar-dialogs.h>
-#include <thunar/thunar-gtk-extensions.h>
-#include <thunar/thunar-icon-factory.h>
-#include <thunar/thunar-icon-renderer.h>
-#include <thunar/thunar-menu.h>
-#include <thunar/thunar-private.h>
-#include <thunar/thunar-properties-dialog.h>
-#include <thunar/thunar-renamer-dialog.h>
-#include <thunar/thunar-renamer-model.h>
-#include <thunar/thunar-renamer-progress.h>
+#include "thunar/thunar-abstract-dialog.h"
+#include "thunar/thunar-action-manager.h"
+#include "thunar/thunar-application.h"
+#include "thunar/thunar-dialogs.h"
+#include "thunar/thunar-gtk-extensions.h"
+#include "thunar/thunar-icon-factory.h"
+#include "thunar/thunar-icon-renderer.h"
+#include "thunar/thunar-menu.h"
+#include "thunar/thunar-preferences.h"
+#include "thunar/thunar-private.h"
+#include "thunar/thunar-properties-dialog.h"
+#include "thunar/thunar-renamer-dialog.h"
+#include "thunar/thunar-renamer-model.h"
+#include "thunar/thunar-renamer-progress.h"
+#include "thunar/thunar-util.h"
 
 
 
@@ -134,16 +136,19 @@ static void        thunar_renamer_dialog_row_activated         (GtkTreeView
                                                                 ThunarRenamerDialog      *renamer_dialog);
 static void        thunar_renamer_dialog_selection_changed     (GtkTreeSelection         *selection,
                                                                 ThunarRenamerDialog      *renamer_dialog);
-static ThunarFile *thunar_renamer_dialog_get_current_directory (ThunarRenamerDialog *renamer_dialog);
-static void        thunar_renamer_dialog_set_current_directory (ThunarRenamerDialog *renamer_dialog,
-                                                                ThunarFile          *current_directory);
-static GList      *thunar_renamer_dialog_get_selected_files    (ThunarRenamerDialog *renamer_dialog);
-static gboolean    thunar_renamer_dialog_get_standalone        (ThunarRenamerDialog *renamer_dialog);
-static void        thunar_renamer_dialog_set_standalone        (ThunarRenamerDialog *renamer_dialog,
-                                                                gboolean             fixed);
-static GtkWidget  *thunar_renamer_dialog_append_menu_item      (ThunarRenamerDialog *renamer_dialog,
-                                                                GtkMenuShell        *menu,
-                                                                ThunarRenamerAction  action);
+static ThunarFile *thunar_renamer_dialog_get_current_directory (ThunarRenamerDialog      *renamer_dialog);
+static void        thunar_renamer_dialog_set_current_directory (ThunarRenamerDialog      *renamer_dialog,
+                                                                ThunarFile               *current_directory);
+static GList      *thunar_renamer_dialog_get_selected_files    (ThunarRenamerDialog      *renamer_dialog);
+static gboolean    thunar_renamer_dialog_get_standalone        (ThunarRenamerDialog      *renamer_dialog);
+static void        thunar_renamer_dialog_set_standalone        (ThunarRenamerDialog      *renamer_dialog,
+                                                                gboolean                  fixed);
+static GtkWidget  *thunar_renamer_dialog_append_menu_item      (ThunarRenamerDialog      *renamer_dialog,
+                                                                GtkMenuShell             *menu,
+                                                                ThunarRenamerAction       action);
+static gboolean    thunar_renamer_configure_event              (GtkWidget                *widget,
+                                                                GdkEventConfigure        *event);
+static void        thunar_renamer_save_geometry_timer_destroy  (gpointer                  user_data);                                                                
 
 
 
@@ -163,7 +168,6 @@ struct _ThunarRenamerDialog
 
   ThunarActionManager *action_mgr;
 
-  GtkWidget           *cancel_button;
   GtkWidget           *close_button;
 
   GtkWidget           *tree_view;
@@ -184,6 +188,9 @@ struct _ThunarRenamerDialog
 
   /* TRUE while drop highlighting */
   gboolean             drag_highlighted;
+  
+  /* support to remember window geometry */
+  guint                save_geometry_timer_id;
 };
 
 
@@ -232,6 +239,7 @@ thunar_renamer_dialog_class_init (ThunarRenamerDialogClass *klass)
   gtkwidget_class = GTK_WIDGET_CLASS (klass);
   gtkwidget_class->realize = thunar_renamer_dialog_realize;
   gtkwidget_class->unrealize = thunar_renamer_dialog_unrealize;
+  gtkwidget_class->configure_event = thunar_renamer_configure_event;
 
   gtkdialog_class = GTK_DIALOG_CLASS (klass);
   gtkdialog_class->response = thunar_renamer_dialog_response;
@@ -302,6 +310,7 @@ static void
 thunar_renamer_dialog_init (ThunarRenamerDialog *renamer_dialog)
 {
   ThunarxProviderFactory *provider_factory;
+  ThunarPreferences      *preferences;
   GtkTreeViewColumn      *column;
   GtkTreeSelection       *selection;
   GtkCellRenderer        *renderer;
@@ -329,6 +338,9 @@ thunar_renamer_dialog_init (ThunarRenamerDialog *renamer_dialog)
   GList                  *renamers = NULL;
   GList                  *lp;
   gint                    active = 0;
+  gint                    last_dialog_width;
+  gint                    last_dialog_height;
+  gint                    last_dialog_maximized;
   guint                   n;
 
   /* allocate a new bulk rename model */
@@ -351,29 +363,32 @@ thunar_renamer_dialog_init (ThunarRenamerDialog *renamer_dialog)
   g_list_free (providers);
 
   /* initialize the dialog */
-  gtk_window_set_default_size (GTK_WINDOW (renamer_dialog), 510, 490);
+  preferences = thunar_preferences_get ();
+  g_object_get (G_OBJECT (preferences),
+                "last-renamer-dialog-width", &last_dialog_width,
+                "last-renamer-dialog-height", &last_dialog_height,
+                "last-renamer-dialog-maximized", &last_dialog_maximized,
+                NULL);
+  g_object_unref(preferences);
+  gtk_window_set_default_size (GTK_WINDOW (renamer_dialog), last_dialog_width, last_dialog_height);
+  
+  /* restore the maxized state of the dialog */
+  if (G_UNLIKELY (last_dialog_maximized))
+    gtk_window_maximize (GTK_WINDOW (renamer_dialog));
+  
   gtk_window_set_title (GTK_WINDOW (renamer_dialog), _("Rename Multiple Files"));
 
-  /* add the Cancel/Close buttons */
-  renamer_dialog->cancel_button = gtk_dialog_add_button (GTK_DIALOG (renamer_dialog), _("_Cancel"), GTK_RESPONSE_CANCEL);
+  /* add the "Close" button */
   renamer_dialog->close_button = gtk_dialog_add_button (GTK_DIALOG (renamer_dialog), _("_Close"), GTK_RESPONSE_DELETE_EVENT);
-  gtk_widget_hide (renamer_dialog->close_button);
+  gtk_widget_set_tooltip_text (renamer_dialog->close_button, _("Close this window, cancel further changes."));
 
-  /* add the "Done" button */
-  button = gtk_dialog_add_button (GTK_DIALOG (renamer_dialog), _("_Done"), GTK_RESPONSE_ACCEPT);
-  g_object_bind_property (G_OBJECT (renamer_dialog->model), "can-rename",
-                          G_OBJECT (button),                "sensitive",
-                          G_BINDING_SYNC_CREATE);
-  gtk_dialog_set_default_response (GTK_DIALOG (renamer_dialog), GTK_RESPONSE_ACCEPT);
-  gtk_widget_set_tooltip_text (button, _("Click here to actually rename the files listed above to their new names and close the window."));
-
-  /* add the "Apply" button */
-  button = gtk_dialog_add_button (GTK_DIALOG (renamer_dialog), _("_Apply"), GTK_RESPONSE_APPLY);
+  /* add the "Rename" button */
+  button = gtk_dialog_add_button (GTK_DIALOG (renamer_dialog), _("_Rename"), GTK_RESPONSE_APPLY);
   g_object_bind_property (G_OBJECT (renamer_dialog->model), "can-rename",
                           G_OBJECT (button),                "sensitive",
                           G_BINDING_SYNC_CREATE);
   gtk_dialog_set_default_response (GTK_DIALOG (renamer_dialog), GTK_RESPONSE_APPLY);
-  gtk_widget_set_tooltip_text (button, _("Click here to actually rename the files listed above to their new names and keep the window open."));
+  gtk_widget_set_tooltip_text (button, _("Rename items listed above, keep this window open."));
 
   /* setup the action manager support for this dialog */
   renamer_dialog->action_mgr = g_object_new (THUNAR_TYPE_ACTION_MANAGER, "widget", GTK_WIDGET (renamer_dialog), NULL);
@@ -639,6 +654,10 @@ thunar_renamer_dialog_dispose (GObject *object)
 {
   ThunarRenamerDialog *renamer_dialog = THUNAR_RENAMER_DIALOG (object);
 
+  /* destroy the save geometry timer source */
+  if (G_UNLIKELY (renamer_dialog->save_geometry_timer_id != 0))
+    g_source_remove (renamer_dialog->save_geometry_timer_id);
+
   /* reset the "current-directory" property */
   thunar_renamer_dialog_set_current_directory (renamer_dialog, NULL);
 
@@ -767,12 +786,8 @@ thunar_renamer_dialog_response (GtkDialog *dialog,
   g_object_ref (G_OBJECT (dialog));
 
   /* check if we should rename */
-  if (G_LIKELY (response == GTK_RESPONSE_ACCEPT  || response == GTK_RESPONSE_APPLY))
+  if (G_LIKELY (response == GTK_RESPONSE_APPLY))
     {
-      /* hide the close button and show the cancel button */
-      gtk_widget_show (renamer_dialog->cancel_button);
-      gtk_widget_hide (renamer_dialog->close_button);
-
       /* save the current settings */
       thunar_renamer_dialog_save (renamer_dialog);
 
@@ -810,20 +825,6 @@ thunar_renamer_dialog_response (GtkDialog *dialog,
 
       /* release the pairs */
       thunar_renamer_pair_list_free (pair_list);
-
-      /* check if we're in standalone mode */
-      if (thunar_renamer_dialog_get_standalone (renamer_dialog) || response == GTK_RESPONSE_APPLY)
-        {
-          /* hide the cancel button again and display the close button */
-          gtk_widget_hide (renamer_dialog->cancel_button);
-          gtk_widget_show (renamer_dialog->close_button);
-        }
-      else
-        {
-          /* hide and destroy the dialog window */
-          gtk_widget_hide (GTK_WIDGET (dialog));
-          gtk_widget_destroy (GTK_WIDGET (dialog));
-        }
     }
   else if (thunar_renamer_progress_running (THUNAR_RENAMER_PROGRESS (renamer_dialog->progress)))
     {
@@ -1806,20 +1807,6 @@ thunar_renamer_dialog_set_standalone (ThunarRenamerDialog *renamer_dialog,
       /* change title to reflect the standalone status */
       gtk_window_set_title (GTK_WINDOW (renamer_dialog), standalone ? _("Bulk Rename - Rename Multiple Files") : _("Rename Multiple Files"));
 
-      /* display the appropriate Cancel/Close button */
-      if (G_UNLIKELY (standalone))
-        {
-          /* standalone has Close button by default */
-          gtk_widget_hide (renamer_dialog->cancel_button);
-          gtk_widget_show (renamer_dialog->close_button);
-        }
-      else
-        {
-          /* else, Cancel button is default */
-          gtk_widget_show (renamer_dialog->cancel_button);
-          gtk_widget_hide (renamer_dialog->close_button);
-        }
-
       /* notify listeners */
       g_object_notify (G_OBJECT (renamer_dialog), "standalone");
     }
@@ -1923,3 +1910,40 @@ thunar_show_renamer_dialog (gpointer     parent,
 }
 
 
+
+static gboolean
+thunar_renamer_configure_event (GtkWidget         *widget,
+                                GdkEventConfigure *event)
+{
+  ThunarRenamerDialog *dialog = THUNAR_RENAMER_DIALOG (widget);
+  GtkAllocation        widget_allocation;
+
+  gtk_widget_get_allocation (widget, &widget_allocation);
+
+  /* check if we have a new dimension here */
+  if (widget_allocation.width != event->width || widget_allocation.height != event->height)
+    {
+      /* drop any previous timer source */
+      if (dialog->save_geometry_timer_id != 0)
+        g_source_remove (dialog->save_geometry_timer_id);
+
+      /* check if we should schedule another save timer */
+      if (gtk_widget_get_visible (widget))
+        {
+          /* save the geometry one second after the last configure event */
+          dialog->save_geometry_timer_id = g_timeout_add_seconds_full (G_PRIORITY_LOW, 1, thunar_util_save_geometry_timer,
+                                                                       dialog, thunar_renamer_save_geometry_timer_destroy);
+        }
+    }
+
+  /* let Gtk+ handle the configure event */
+  return (*GTK_WIDGET_CLASS (thunar_renamer_dialog_parent_class)->configure_event) (widget, event);
+}
+
+
+
+static void
+thunar_renamer_save_geometry_timer_destroy (gpointer user_data)
+{
+  THUNAR_RENAMER_DIALOG (user_data)->save_geometry_timer_id = 0;
+}
