@@ -19,7 +19,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #ifdef HAVE_MEMORY_H
@@ -29,11 +29,11 @@
 #include <string.h>
 #endif
 
-#include <thunar/thunar-file-monitor.h>
-#include <thunar/thunar-gobject-extensions.h>
-#include <thunar/thunar-private.h>
-#include <thunar/thunar-renamer-model.h>
-#include <thunar/thunar-util.h>
+#include "thunar/thunar-file-monitor.h"
+#include "thunar/thunar-gobject-extensions.h"
+#include "thunar/thunar-private.h"
+#include "thunar/thunar-renamer-model.h"
+#include "thunar/thunar-util.h"
 
 
 
@@ -498,14 +498,14 @@ thunar_renamer_model_get_value (GtkTreeModel *tree_model,
       if (G_LIKELY (item->name != NULL))
         g_value_set_string (value, item->name);
       else if (item->conflict)
-        g_value_set_static_string (value, thunar_file_get_display_name (item->file));
+        g_value_set_static_string (value, thunar_file_get_basename (item->file));
       else
         g_value_set_static_string (value, "");
       break;
 
     case THUNAR_RENAMER_MODEL_COLUMN_OLDNAME:
       g_value_init (value, G_TYPE_STRING);
-      g_value_set_static_string (value, thunar_file_get_display_name (item->file));
+      g_value_set_static_string (value, thunar_file_get_basename (item->file));
       break;
 
     default:
@@ -771,8 +771,8 @@ thunar_renamer_model_conflict_item (ThunarRenamerModel     *renamer_model,
 
       /* check if this other item conflicts with the item in question (can only conflict if in same directory) */
       if (trm_same_directory (item->file, oitem->file)
-          && ((item->name != NULL && oitem->name == NULL && strcmp (item->name, thunar_file_get_display_name (oitem->file)) == 0)
-           || (item->name == NULL && oitem->name != NULL && strcmp (thunar_file_get_display_name (item->file), oitem->name) == 0)
+          && ((item->name != NULL && oitem->name == NULL && strcmp (item->name, thunar_file_get_basename (oitem->file)) == 0)
+           || (item->name == NULL && oitem->name != NULL && strcmp (thunar_file_get_basename (item->file), oitem->name) == 0)
            || (item->name != NULL && oitem->name != NULL && strcmp (item->name, oitem->name) == 0)))
         {
           /* check if the other item is already in conflict state */
@@ -806,7 +806,7 @@ thunar_renamer_model_process_item (ThunarRenamerModel     *renamer_model,
                                    guint                   idx)
 {
   ThunarRenamerMode mode;
-  const gchar      *display_name;
+  const gchar      *file_name;
   const gchar      *dot;
   gchar            *name = NULL;
   gchar            *prefix;
@@ -817,11 +817,11 @@ thunar_renamer_model_process_item (ThunarRenamerModel     *renamer_model,
   if (G_UNLIKELY (renamer_model->renamer == NULL))
     return NULL;
 
-  /* determine the current display name of the file */
-  display_name = thunar_file_get_display_name (item->file);
+  /* determine the current file name of the file */
+  file_name = thunar_file_get_basename (item->file);
 
   /* determine the extension in the filename */
-  dot = thunar_util_str_get_extension (display_name);
+  dot = thunar_util_str_get_extension (file_name);
 
   /* if we don't have a dot, then no "Extension only" rename can take place */
   if (G_LIKELY (dot != NULL || renamer_model->mode != THUNAR_RENAMER_MODE_EXTENSION))
@@ -837,7 +837,7 @@ thunar_renamer_model_process_item (ThunarRenamerModel     *renamer_model,
         {
         case THUNAR_RENAMER_MODE_NAME:
           /* determine the name part of the display name */
-          text = g_strndup (display_name, (dot - display_name));
+          text = g_strndup (file_name, (dot - file_name));
 
           /* determine the new name */
           prefix = thunarx_renamer_process (renamer_model->renamer, THUNARX_FILE_INFO (item->file), text, idx);
@@ -854,7 +854,7 @@ thunar_renamer_model_process_item (ThunarRenamerModel     *renamer_model,
           /* determine the new extension */
           extension = thunarx_renamer_process (renamer_model->renamer, THUNARX_FILE_INFO (item->file), dot + 1, idx);
 
-          prefix = g_strndup (display_name, (dot - display_name) + 1);
+          prefix = g_strndup (file_name, (dot - file_name) + 1);
           name = g_strconcat (prefix, extension, NULL);
           g_free (prefix);
 
@@ -864,7 +864,7 @@ thunar_renamer_model_process_item (ThunarRenamerModel     *renamer_model,
 
         case THUNAR_RENAMER_MODE_BOTH:
           /* determine the new full name */
-          name = thunarx_renamer_process (renamer_model->renamer, THUNARX_FILE_INFO (item->file), display_name, idx);
+          name = thunarx_renamer_process (renamer_model->renamer, THUNARX_FILE_INFO (item->file), file_name, idx);
           break;
 
         default:
@@ -874,7 +874,7 @@ thunar_renamer_model_process_item (ThunarRenamerModel     *renamer_model,
     }
 
   /* check if the new name is equal to the old one */
-  if (g_strcmp0 (name, display_name) == 0)
+  if (g_strcmp0 (name, file_name) == 0)
     {
       /* just return NULL then */
       g_free (name);
@@ -899,8 +899,6 @@ thunar_renamer_model_update_idle (gpointer user_data)
   gchar                  *name;
   GList                  *lp;
 
-THUNAR_THREADS_ENTER
-
   /* don't do anything if the model is frozen */
   if (G_LIKELY (!renamer_model->frozen))
     {
@@ -961,8 +959,6 @@ THUNAR_THREADS_ENTER
         }
     }
 
-THUNAR_THREADS_LEAVE
-
   /* keep the idle source as long as any item changed */
   return changed;
 }
