@@ -19,17 +19,17 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include <gio/gio.h>
 
-#include <thunar/thunar-gio-extensions.h>
-#include <thunar/thunar-gobject-extensions.h>
-#include <thunar/thunar-gdk-extensions.h>
-#include <thunar/thunar-icon-factory.h>
-#include <thunar/thunar-shortcuts-icon-renderer.h>
-#include <thunar/thunar-device.h>
+#include "thunar/thunar-gio-extensions.h"
+#include "thunar/thunar-gobject-extensions.h"
+#include "thunar/thunar-gdk-extensions.h"
+#include "thunar/thunar-icon-factory.h"
+#include "thunar/thunar-shortcuts-icon-renderer.h"
+#include "thunar/thunar-device.h"
 
 
 
@@ -238,9 +238,10 @@ thunar_shortcuts_icon_renderer_render (GtkCellRenderer     *renderer,
         gicon = thunar_device_get_icon (shortcuts_icon_renderer->device);
 
       scale_factor = gtk_widget_get_scale_factor (widget);
-      icon_info = gtk_icon_theme_lookup_by_gicon (icon_theme, gicon, cell_area->width * scale_factor,
-                                                  GTK_ICON_LOOKUP_USE_BUILTIN |
-                                                  GTK_ICON_LOOKUP_FORCE_SIZE);
+      icon_info = gtk_icon_theme_lookup_by_gicon_for_scale (icon_theme, gicon,
+                                                            cell_area->width, scale_factor,
+                                                            GTK_ICON_LOOKUP_USE_BUILTIN |
+                                                            GTK_ICON_LOOKUP_FORCE_SIZE);
       g_object_unref (gicon);
 
       /* try to load the icon */
