@@ -529,10 +529,30 @@ thunar_thumbnail_cache_move_file (ThunarThumbnailCache *cache,
                                   GFile                *source_file,
                                   GFile                *target_file)
 {
+  GFileInfo *file_info;
+  gboolean   is_symlink = TRUE;
+
   _thunar_return_if_fail (THUNAR_IS_THUMBNAIL_CACHE (cache));
   _thunar_return_if_fail (G_IS_FILE (source_file));
   _thunar_return_if_fail (G_IS_FILE (target_file));
 
+  /* For some weird reason, gio will spam criticals, when we dont query is-hidden/is-backup. So lets just querry all standard attributes */
+  file_info = g_file_query_info (target_file, "standard::*", G_FILE_QUERY_INFO_NONE, NULL, NULL);
+
+  if (file_info != NULL)
+    {
+      is_symlink = g_file_info_get_attribute_boolean (file_info, G_FILE_ATTRIBUTE_STANDARD_IS_SYMLINK);
+      g_object_unref (file_info);
+    }
+
+  /* relative symlinks might point to new files */
+  /* So if the file is a symlink, or we are not sure, only clear the old cache */
+  if (is_symlink)
+    {
+      thunar_thumbnail_cache_delete_file (cache, source_file);
+      return;
+    }
+
   /* acquire a cache lock */
   _thumbnail_cache_lock (cache);
 
@@ -572,10 +592,27 @@ thunar_thumbnail_cache_copy_file (ThunarThumbnailCache *cache,
                                   GFile                *source_file,
                                   GFile                *target_file)
 {
+  GFileInfo *file_info;
+  gboolean   is_symlink = TRUE;
+
   _thunar_return_if_fail (THUNAR_IS_THUMBNAIL_CACHE (cache));
   _thunar_return_if_fail (G_IS_FILE (source_file));
   _thunar_return_if_fail (G_IS_FILE (target_file));
 
+  /* For some weird reason, gio will spam criticals, when we dont query is-hidden/is-backup. So lets just querry all standard attributes */
+  file_info = g_file_query_info (target_file, "standard::*", G_FILE_QUERY_INFO_NONE, NULL, NULL);
+
+  if (file_info != NULL)
+    {
+      is_symlink = g_file_info_get_attribute_boolean (file_info, G_FILE_ATTRIBUTE_STANDARD_IS_SYMLINK);
+      g_object_unref (file_info);
+    }
+
+  /* relative symlinks might point to new files */
+  /* So if the file is a symlink, or we are not sure, rather dont copy the cache */
+  if (is_symlink)
+    return;
+
   /* acquire a cache lock */
   _thumbnail_cache_lock (cache);
 
