@@ -95,7 +95,6 @@ typedef enum
 
 
 typedef struct _ThunarThumbnailerJob  ThunarThumbnailerJob;
-typedef struct _ThunarThumbnailerIdle ThunarThumbnailerIdle;
 
 /* Signal identifiers */
 enum
@@ -114,7 +113,7 @@ enum
 
 
 static void                   thunar_thumbnailer_finalize               (GObject                    *object);
-static void                   thunar_thumbnailer_init_thumbnailer_proxy (ThunarThumbnailer          *thumbnailer);
+static gboolean               thunar_thumbnailer_init_thumbnailer_proxy (ThunarThumbnailer          *thumbnailer);
 static gboolean               thunar_thumbnailer_file_is_supported      (ThunarThumbnailer          *thumbnailer,
                                                                          ThunarFile                 *file);
 static void                   thunar_thumbnailer_thumbnailer_finished   (GDBusProxy                 *proxy,
@@ -130,12 +129,6 @@ static void                   thunar_thumbnailer_thumbnailer_ready      (GDBusPr
                                                                          guint32                     handle,
                                                                          const gchar               **uris,
                                                                          ThunarThumbnailer          *thumbnailer);
-static void                   thunar_thumbnailer_idle                   (ThunarThumbnailer          *thumbnailer,
-                                                                         guint                       handle,
-                                                                         ThunarThumbnailerIdleType   type,
-                                                                         const gchar               **uris);
-static gboolean               thunar_thumbnailer_idle_func              (gpointer                    user_data);
-static void                   thunar_thumbnailer_idle_free              (gpointer                    data);
 static void                   thunar_thumbnailer_get_property           (GObject                    *object,
                                                                          guint                       prop_id,
                                                                          GValue                     *value,
@@ -180,9 +173,12 @@ struct _ThunarThumbnailer
 
   /* maximum file size (in bytes) allowed to be thumbnailed */
   guint64     thumbnail_max_file_size;
-  
-  /* IDs of idle functions */
-  GSList     *idles;
+
+  /* Agregated thumbnailing requests per size, to be queued on timeout */
+  ThunarThumbnailerJob* jobs_to_queue[N_THUMBNAIL_SIZES];
+
+  /* Id's of the timeout sources per size, used to agrregate requets */
+  guint                 jobs_to_queue_source_id[N_THUMBNAIL_SIZES];
 };
 
 struct _ThunarThumbnailerJob
@@ -192,8 +188,6 @@ struct _ThunarThumbnailerJob
   /* if this job is cancelled */
   guint              cancelled : 1;
 
-  guint              lazy_checks : 1;
-
   /* data is saved here in case the queueing is delayed */
   /* If this is NULL, the request has been sent off. */
   GList             *files; /* element type: ThunarFile */
@@ -208,15 +202,6 @@ struct _ThunarThumbnailerJob
   ThunarThumbnailSize thumbnail_size;
 };
 
-struct _ThunarThumbnailerIdle
-{
-  ThunarThumbnailerIdleType  type;
-  ThunarThumbnailer          *thumbnailer;
-  guint                       id;
-  gchar                     **uris;
-};
-
-
 static guint thumbnailer_signals[LAST_SIGNAL];
 
 
@@ -413,9 +398,7 @@ thunar_thumbnailer_begin_job (ThunarThumbnailer *thumbnailer,
   GList                 *supported_files = NULL;
   guint                  n;
   guint                  n_items = 0;
-  ThunarFileThumbState   thumb_state;
   const gchar           *thumbnail_path;
-  gint                   request_no;
   ThunarThumbnailSize    thumbnail_size;
 
   if (thumbnailer->proxy_state == THUNAR_THUMBNAILER_PROXY_WAITING)
@@ -425,7 +408,21 @@ thunar_thumbnailer_begin_job (ThunarThumbnailer *thumbnailer,
     }
   else if (thumbnailer->proxy_state == THUNAR_THUMBNAILER_PROXY_FAILED)
     {
-      /* the job has no chance to be completed - ever */
+      /* give another chance to the proxy */
+      g_warning ("Thumbnailer Proxy Failed ... starting attempt to re-initialize");
+      
+      if (thumbnailer->thumbnailer_proxy != NULL)
+        {
+          /* disconnect from the thumbnailer proxy */
+          g_signal_handlers_disconnect_by_data (thumbnailer->thumbnailer_proxy, thumbnailer);
+          thumbnailer->proxy_state = THUNAR_THUMBNAILER_PROXY_WAITING;
+          g_object_unref (thumbnailer->thumbnailer_proxy);
+          thumbnailer->thumbnailer_proxy = NULL;
+        }
+
+      /* Only try once a second, in order to prevent flooding in case the proxy always fails */
+      g_timeout_add_seconds (1, G_SOURCE_FUNC (thunar_thumbnailer_init_thumbnailer_proxy), thumbnailer);
+
       return FALSE;
     }
 
@@ -436,28 +433,13 @@ thunar_thumbnailer_begin_job (ThunarThumbnailer *thumbnailer,
    * processed (and awaiting to be refreshed) */
   for (lp = job->files; lp != NULL; lp = lp->next)
     {
-      /* the icon factory only loads icons for regular files and folders */
-      if (!thunar_file_is_regular (lp->data) && !thunar_file_is_directory (lp->data))
+      /* the icon factory only loads icons for regular files, symlinks and folders */
+      if (!thunar_file_is_regular (lp->data) && !thunar_file_is_directory (lp->data) && !thunar_file_is_symlink (lp->data))
         {
-          thunar_file_set_thumb_state (lp->data, THUNAR_FILE_THUMB_STATE_NONE, thumbnail_size);
+          thunar_file_update_thumbnail (lp->data, THUNAR_FILE_THUMB_STATE_NONE, thumbnail_size);
           continue;
         }
-
-      /* get the current thumb state */
-      thumb_state = thunar_file_get_thumb_state (lp->data, thumbnail_size);
-
-      if (job->lazy_checks)
-        {
-          /* in lazy mode, don't both for files that have already
-           * been loaded or are not supported */
-          if (thumb_state == THUNAR_FILE_THUMB_STATE_NONE
-              || thumb_state == THUNAR_FILE_THUMB_STATE_READY)
-            continue;
-        }
-
-      /* check if the file is supported, assume it is when the state was ready previously */
-      if (thumb_state == THUNAR_FILE_THUMB_STATE_READY
-          || thunar_thumbnailer_file_is_supported (thumbnailer, lp->data))
+      if (thunar_thumbnailer_file_is_supported (thumbnailer, lp->data))
         {
           guint max_size = thumbnailer->thumbnail_max_file_size;
 
@@ -476,9 +458,9 @@ thunar_thumbnailer_begin_job (ThunarThumbnailer *thumbnailer,
 
           /* test if a thumbnail can be found */
           if (thumbnail_path != NULL && g_file_test (thumbnail_path, G_FILE_TEST_EXISTS))
-            thunar_file_set_thumb_state (lp->data, THUNAR_FILE_THUMB_STATE_READY, thumbnail_size);
+            thunar_file_update_thumbnail (lp->data, THUNAR_FILE_THUMB_STATE_READY, thumbnail_size);
           else
-            thunar_file_set_thumb_state (lp->data, THUNAR_FILE_THUMB_STATE_NONE, thumbnail_size);
+            thunar_file_update_thumbnail (lp->data, THUNAR_FILE_THUMB_STATE_NONE, thumbnail_size);
         }
     }
 
@@ -492,9 +474,6 @@ thunar_thumbnailer_begin_job (ThunarThumbnailer *thumbnailer,
       /* fill URI and MIME hint arrays with items from the wait queue */
       for (lp = supported_files, n = 0; lp != NULL; lp = lp->next, ++n)
         {
-          /* set the thumbnail state to loading */
-          thunar_file_set_thumb_state (lp->data, THUNAR_FILE_THUMB_STATE_LOADING, thumbnail_size);
-
           /* save URI and MIME hint in the arrays */
           uris[n] = thunar_file_dup_uri (lp->data);
           mime_hints[n] = thunar_file_get_content_type (lp->data);
@@ -504,17 +483,6 @@ thunar_thumbnailer_begin_job (ThunarThumbnailer *thumbnailer,
       uris[n] = NULL;
       mime_hints[n] = NULL;
 
-      /* queue a thumbnail request for the URIs from the wait queue */
-      /* compute the next request ID, making sure it's never 0 */
-      request_no = thumbnailer->last_request + 1;
-      request_no = MAX (request_no, 1);
-
-      /* remember the ID for the next request */
-      thumbnailer->last_request = request_no;
-
-      /* save the request number */
-      job->request = request_no;
-
       /* increase the reference count while the dbus call is running */
       g_object_ref (thumbnailer);
 
@@ -553,12 +521,22 @@ thunar_thumbnailer_init (ThunarThumbnailer *thumbnailer)
 {
   g_mutex_init (&thumbnailer->lock);
 
+  _thumbnailer_lock (thumbnailer);
+
   /* initialize the proxies */
   thunar_thumbnailer_init_thumbnailer_proxy (thumbnailer);
 
+  _thumbnailer_unlock (thumbnailer);
+
   /* grab a reference on the preferences */
   thumbnailer->preferences = thunar_preferences_get ();
 
+  for (gint i = 0; i < N_THUMBNAIL_SIZES; i++)
+    {
+      thumbnailer->jobs_to_queue[i] = NULL;
+      thumbnailer->jobs_to_queue_source_id[i] = 0;
+    }
+
   g_object_bind_property (G_OBJECT (thumbnailer->preferences),
                           "misc-thumbnail-max-file-size",
                           G_OBJECT (thumbnailer),
@@ -572,28 +550,27 @@ static void
 thunar_thumbnailer_finalize (GObject *object)
 {
   ThunarThumbnailer     *thumbnailer = THUNAR_THUMBNAILER (object);
-  ThunarThumbnailerIdle *idle;
-  GSList                *lp;
 
   /* acquire the thumbnailer lock */
   _thumbnailer_lock (thumbnailer);
 
+  for (gint i = 0; i < N_THUMBNAIL_SIZES; i++)
+    {
+      if (thumbnailer->jobs_to_queue_source_id[i] != 0)
+      {
+        g_source_remove (thumbnailer->jobs_to_queue_source_id[i]);
+        thunar_thumbnailer_free_job (thumbnailer->jobs_to_queue[i]);
+        thumbnailer->jobs_to_queue[i] = NULL;
+        thumbnailer->jobs_to_queue_source_id[i] = 0;
+      }
+    }
+
   if (thumbnailer->thumbnailer_proxy != NULL)
     {
       /* disconnect from the thumbnailer proxy */
-      g_signal_handlers_disconnect_matched (thumbnailer->thumbnailer_proxy,
-                                            G_SIGNAL_MATCH_DATA, 0, 0,
-                                            NULL, NULL, thumbnailer);
+      g_signal_handlers_disconnect_by_data (thumbnailer->thumbnailer_proxy, thumbnailer);
     }
 
-  /* abort all pending idle functions */
-  for (lp = thumbnailer->idles; lp != NULL; lp = lp->next)
-    {
-      idle = lp->data;
-      g_source_remove (idle->id);
-    }
-  g_slist_free (thumbnailer->idles);
-
   /* remove all jobs */
   g_slist_free_full (thumbnailer->jobs, (GDestroyNotify)thunar_thumbnailer_free_job);
 
@@ -681,11 +658,15 @@ thunar_thumbnailer_received_supported_types (ThunarThumbnailerDBus  *proxy,
       g_clear_error (&error);
 
       thumbnailer->proxy_state = THUNAR_THUMBNAILER_PROXY_FAILED;
-      g_object_unref (proxy);
+
+      /* disconnect from the thumbnailer proxy */
+      g_signal_handlers_disconnect_by_data (thumbnailer->thumbnailer_proxy, thumbnailer);
+
+      g_object_unref (thumbnailer->thumbnailer_proxy);
+      thumbnailer->thumbnailer_proxy = NULL;
 
       _thumbnailer_unlock (thumbnailer);
 
-      g_object_unref (thumbnailer);
       return;
     }
 
@@ -721,7 +702,6 @@ thunar_thumbnailer_received_supported_types (ThunarThumbnailerDBus  *proxy,
     }
 
   thumbnailer->proxy_state = THUNAR_THUMBNAILER_PROXY_AVAILABLE;
-  thumbnailer->thumbnailer_proxy = proxy;
 
   /* now start delayed jobs */
   for (lp = thumbnailer->jobs; lp; lp = lp->next)
@@ -737,8 +717,6 @@ thunar_thumbnailer_received_supported_types (ThunarThumbnailerDBus  *proxy,
   g_clear_error (&error);
 
   _thumbnailer_unlock (thumbnailer);
-
-  g_object_unref (thumbnailer);
 }
 
 
@@ -750,11 +728,10 @@ thunar_thumbnailer_proxy_created (GObject       *object,
 {
   ThunarThumbnailer     *thumbnailer = THUNAR_THUMBNAILER (userdata);
   GError                *error = NULL;
-  ThunarThumbnailerDBus *proxy;
 
-  proxy = thunar_thumbnailer_dbus_proxy_new_finish (result, &error);
+  thumbnailer->thumbnailer_proxy = thunar_thumbnailer_dbus_proxy_new_finish (result, &error);
 
-  if (!proxy)
+  if (thumbnailer->thumbnailer_proxy == NULL)
     {
       _thumbnailer_lock (thumbnailer);
 
@@ -767,18 +744,26 @@ thunar_thumbnailer_proxy_created (GObject       *object,
 
       _thumbnailer_unlock (thumbnailer);
 
-      g_object_unref (thumbnailer);
-
       return;
     }
 
-  /* setup signals */
-  g_signal_connect (proxy, "error",
+  /* More detailed information about the Thumbnailer sigbnals can be found here */
+  /* https://wiki.gnome.org/Attic/DraftThumbnailerSpec */
+
+  /* 'error' is signaled when thumnailing failed for some uris */
+  /* Note that 'finished' will still be signaled after the whole thumbnailing request finished */
+  g_signal_connect (thumbnailer->thumbnailer_proxy, "error",
                     G_CALLBACK (thunar_thumbnailer_thumbnailer_error), thumbnailer);
-  g_signal_connect (proxy, "finished",
+
+  /* 'finished' is signaled after a thumbnailing request finished .. no matter if sucessfull or not */
+  g_signal_connect (thumbnailer->thumbnailer_proxy, "finished",
                     G_CALLBACK (thunar_thumbnailer_thumbnailer_finished), thumbnailer);
-  g_signal_connect (proxy, "ready",
-                    G_CALLBACK (thunar_thumbnailer_thumbnailer_ready), thumbnailer);
+
+  /* 'ready' is signaled when thumnailing was sucessfull for some uris */
+  /* Note that 'finished' will still be signaled after the whole thumbnailing request finished */
+  g_signal_connect (thumbnailer->thumbnailer_proxy, "ready",
+                   G_CALLBACK (thunar_thumbnailer_thumbnailer_ready), thumbnailer);
+
 
   /* begin retrieving supported file types */
 
@@ -791,7 +776,7 @@ thunar_thumbnailer_proxy_created (GObject       *object,
                                                   (GDestroyNotify) g_ptr_array_unref);
 
   /* request the supported types from the thumbnailer D-Bus service. */
-  thunar_thumbnailer_dbus_call_get_supported (proxy, NULL,
+  thunar_thumbnailer_dbus_call_get_supported (thumbnailer->thumbnailer_proxy, NULL,
                                               (GAsyncReadyCallback)thunar_thumbnailer_received_supported_types,
                                               thumbnailer);
 
@@ -800,11 +785,9 @@ thunar_thumbnailer_proxy_created (GObject       *object,
 
 
 
-static void
+static gboolean
 thunar_thumbnailer_init_thumbnailer_proxy (ThunarThumbnailer *thumbnailer)
 {
-  _thumbnailer_lock (thumbnailer);
-
   thumbnailer->thumbnailer_proxy = NULL;
   thumbnailer->proxy_state = THUNAR_THUMBNAILER_PROXY_WAITING;
 
@@ -817,7 +800,7 @@ thunar_thumbnailer_init_thumbnailer_proxy (ThunarThumbnailer *thumbnailer)
                                              (GAsyncReadyCallback)thunar_thumbnailer_proxy_created,
                                              thumbnailer);
 
-  _thumbnailer_unlock (thumbnailer);
+  return G_SOURCE_REMOVE;
 }
 
 
@@ -839,13 +822,25 @@ thunar_thumbnailer_file_is_supported (ThunarThumbnailer *thumbnailer,
   _thunar_return_val_if_fail (thumbnailer->supported != NULL, FALSE);
 
   /* determine the content type of the passed file */
-  content_type = thunar_file_get_content_type (file);
+  if (thunar_file_is_symlink (file))
+    {
+      GFile *link_target;
+
+      link_target = thunar_g_file_resolve_symlink (thunar_file_get_file (file));
+      if (link_target == NULL)
+        return FALSE;
+
+      content_type = thunar_g_file_get_content_type (link_target);
+      g_object_unref (link_target);
+   }
+  else
+    content_type = thunar_file_get_content_type (file);
 
   /* abort if the content type is unknown */
   if (content_type == NULL)
     return FALSE;
 
-  /* lazy lookup the content type, no difficult parent type matching here */
+  /* lookup the content type, no difficult parent type matching here */
   schemes_array = g_hash_table_lookup (thumbnailer->supported, content_type);
   if (schemes_array != NULL)
     {
@@ -872,14 +867,39 @@ thunar_thumbnailer_thumbnailer_error (GDBusProxy        *proxy,
                                       const gchar       *message,
                                       ThunarThumbnailer *thumbnailer)
 {
+  GFile                *gfile;
+  ThunarFile           *file;
+  ThunarThumbnailerJob *job;
+
   _thunar_return_if_fail (G_IS_DBUS_PROXY (proxy));
   _thunar_return_if_fail (THUNAR_IS_THUMBNAILER (thumbnailer));
 
-  /* check if we have any ready URIs */
-  thunar_thumbnailer_idle (thumbnailer,
-                           handle,
-                           THUNAR_THUMBNAILER_IDLE_ERROR,
-                           uris);
+  _thumbnailer_lock (thumbnailer);
+  for (GSList *lp = thumbnailer->jobs; lp != NULL; lp = lp->next)
+    {
+      job = lp->data;
+      if (job->handle != handle)
+        {
+          for (const gchar **uri = uris; *uri != NULL; ++uri)
+            {
+              /* look up the corresponding ThunarFile from the cache */
+              gfile = g_file_new_for_uri (*uri);
+              file = thunar_file_cache_lookup (gfile);
+              g_object_unref (gfile);
+
+              /* check if we have a file for this URI in the cache */
+              if (file != NULL)
+                {
+                  /* tell everybody we're done here */
+                  thunar_file_update_thumbnail (file, THUNAR_FILE_THUMB_STATE_NONE, job->thumbnail_size);
+                  g_debug ("Failed to generate thumbnail for '%s': Error Code: %i - Error: %s\n", thunar_file_get_basename (file), code, message);
+                  g_object_unref (file);
+                }
+            }
+          break;
+        }
+    }
+   _thumbnailer_unlock (thumbnailer);
 }
 
 
@@ -890,13 +910,36 @@ thunar_thumbnailer_thumbnailer_ready (GDBusProxy        *proxy,
                                       const gchar      **uris,
                                       ThunarThumbnailer *thumbnailer)
 {
+  GFile                *gfile;
+  ThunarFile           *file;
+  ThunarThumbnailerJob *job;
+
   _thunar_return_if_fail (G_IS_DBUS_PROXY (proxy));
   _thunar_return_if_fail (THUNAR_IS_THUMBNAILER (thumbnailer));
 
-  thunar_thumbnailer_idle (thumbnailer,
-                           handle,
-                           THUNAR_THUMBNAILER_IDLE_READY,
-                           uris);
+
+  _thumbnailer_lock (thumbnailer);
+  for (GSList *lp = thumbnailer->jobs; lp != NULL; lp = lp->next)
+    {
+      job = lp->data;
+      if (job->handle == handle)
+        {
+          for (const gchar **uri = uris; *uri != NULL; ++uri)
+            {
+              /* look up the corresponding ThunarFile from the cache */
+              gfile = g_file_new_for_uri (*uri);
+              file = thunar_file_cache_lookup (gfile);
+              g_object_unref (gfile);
+
+              if (file != NULL)
+                {
+                    thunar_file_update_thumbnail (file, THUNAR_FILE_THUMB_STATE_READY, job->thumbnail_size);
+                    g_object_unref (file);
+                }
+            }
+        }
+    }
+      _thumbnailer_unlock (thumbnailer);
 }
 
 
@@ -908,7 +951,6 @@ thunar_thumbnailer_thumbnailer_finished (GDBusProxy        *proxy,
 {
   ThunarThumbnailerJob *job;
   GSList               *lp;
-  GList                *files_to_reload = NULL;
 
   _thunar_return_if_fail (G_IS_DBUS_PROXY (proxy));
   _thunar_return_if_fail (THUNAR_IS_THUMBNAILER (thumbnailer));
@@ -933,9 +975,6 @@ thunar_thumbnailer_thumbnailer_finished (GDBusProxy        *proxy,
           /* tell everybody we're done here */
           g_signal_emit (G_OBJECT (thumbnailer), thumbnailer_signals[REQUEST_FINISHED], 0, job->request);
 
-          for (GList *file_lp = job->files; file_lp != NULL; file_lp = file_lp->next)
-            files_to_reload = g_list_append (files_to_reload, g_object_ref (file_lp->data));
-
           /* remove job from the list */
           thumbnailer->jobs = g_slist_delete_link (thumbnailer->jobs, lp);
 
@@ -945,144 +984,6 @@ thunar_thumbnailer_thumbnailer_finished (GDBusProxy        *proxy,
     }
 
   _thumbnailer_unlock (thumbnailer);
-
-  /* Do the reload outside of teh critical section in order to prevent a deadlock */
-  for (GList *file_lp = files_to_reload; file_lp != NULL; file_lp = file_lp->next)
-    thunar_file_reload (THUNAR_FILE (file_lp->data));
-  
-  if (files_to_reload != NULL)
-    g_list_free_full (files_to_reload, g_object_unref);
-}
-
-
-
-static void
-thunar_thumbnailer_idle (ThunarThumbnailer          *thumbnailer,
-                         guint                       handle,
-                         ThunarThumbnailerIdleType   type,
-                         const gchar               **uris)
-{
-  GSList                *lp;
-  ThunarThumbnailerIdle *idle;
-  ThunarThumbnailerJob  *job;
-
-  /* leave if there are no uris */
-  if (G_UNLIKELY (uris == NULL))
-    return;
-
-  if (handle == 0)
-    {
-      g_printerr ("ThunarThumbnailer: got 0 handle (Error or Ready)\n");
-      return;
-    }
-
-  _thumbnailer_lock (thumbnailer);
-
-  /* look for the job so we don't emit unknown handles, the reason
-   * we do this is when you have multiple windows opened, you don't
-   * want each window (because they all have a connection to the
-   * same proxy) emit the file change, only the window that requested
-   * the data */
-  for (lp = thumbnailer->jobs; lp != NULL; lp = lp->next)
-    {
-      job = lp->data;
-
-      if (job->handle == handle)
-        {
-          /* allocate a new idle struct */
-          idle = g_slice_new0 (ThunarThumbnailerIdle);
-          idle->type = type;
-          idle->thumbnailer = thumbnailer;
-
-          /* copy the URI array because we need it in the idle function */
-          idle->uris = g_strdupv ((gchar **)uris);
-
-          /* remember the idle struct because we might have to remove it in finalize() */
-          thumbnailer->idles = g_slist_prepend (thumbnailer->idles, idle);
-
-          /* call the idle function when we have the time */
-          idle->id = g_idle_add_full (G_PRIORITY_LOW,
-                                      thunar_thumbnailer_idle_func, idle,
-                                      thunar_thumbnailer_idle_free);
-
-          break;
-        }
-    }
-
-  _thumbnailer_unlock (thumbnailer);
-}
-
-
-
-static gboolean
-thunar_thumbnailer_idle_func (gpointer user_data)
-{
-  ThunarThumbnailerIdle *idle = user_data;
-  ThunarFile            *file;
-  GFile                 *gfile;
-  guint                  n;
-
-  _thunar_return_val_if_fail (idle != NULL, FALSE);
-  _thunar_return_val_if_fail (THUNAR_IS_THUMBNAILER (idle->thumbnailer), FALSE);
-
-  /* iterate over all failed URIs */
-  for (n = 0; idle->uris != NULL && idle->uris[n] != NULL; ++n)
-    {
-      /* look up the corresponding ThunarFile from the cache */
-      gfile = g_file_new_for_uri (idle->uris[n]);
-      file = thunar_file_cache_lookup (gfile);
-      g_object_unref (gfile);
-
-      /* check if we have a file for this URI in the cache */
-      if (file != NULL)
-        {
-          if (idle->type == THUNAR_THUMBNAILER_IDLE_ERROR)
-            {
-              /* set thumbnail state to none unless the thumbnail has already been created.
-               * This is to prevent race conditions with the other idle functions */
-              if (thunar_file_get_thumb_state (file, idle->thumbnailer->thumbnail_size) != THUNAR_FILE_THUMB_STATE_READY)
-                thunar_file_set_thumb_state (file, THUNAR_FILE_THUMB_STATE_NONE, idle->thumbnailer->thumbnail_size);
-            }
-          else if (idle->type == THUNAR_THUMBNAILER_IDLE_READY)
-            {
-              /* set thumbnail state to ready - we now have a thumbnail */
-              thunar_file_set_thumb_state (file, THUNAR_FILE_THUMB_STATE_READY, idle->thumbnailer->thumbnail_size);
-            }
-          else
-            {
-              _thunar_assert_not_reached ();
-            }
-          g_object_unref (file);
-        }
-    }
-
-  /* remove the idle struct */
-  _thumbnailer_lock (idle->thumbnailer);
-  idle->thumbnailer->idles = g_slist_remove (idle->thumbnailer->idles, idle);
-  _thumbnailer_unlock (idle->thumbnailer);
-
-  /* remove the idle source, which also destroys the idle struct */
-  return FALSE;
-}
-
-
-
-static void
-thunar_thumbnailer_idle_free (gpointer data)
-{
-  ThunarThumbnailerIdle *idle = data;
-
-  _thunar_return_if_fail (idle != NULL);
-
-  /* free the URI array if necessary */
-  if (idle->type == THUNAR_THUMBNAILER_IDLE_READY
-      || idle->type == THUNAR_THUMBNAILER_IDLE_ERROR)
-    {
-      g_strfreev (idle->uris);
-    }
-
-  /* free the struct */
-  g_slice_free (ThunarThumbnailerIdle, idle);
 }
 
 
@@ -1114,66 +1015,95 @@ thunar_thumbnailer_get (void)
 
 
 
-gboolean
-thunar_thumbnailer_queue_file (ThunarThumbnailer  *thumbnailer,
-                               ThunarFile         *file,
-                               guint              *request,
-                               ThunarThumbnailSize size)
+static gboolean
+thunar_thumbnailer_queue_job_after_timeout (gpointer user_data)
 {
-  GList files;
+  ThunarThumbnailerJob *job = user_data;
+  ThunarThumbnailer    *thumbnailer = THUNAR_THUMBNAILER (job->thumbnailer);
+  ThunarThumbnailSize   thumbnail_size = job->thumbnail_size;
+  gboolean              success = FALSE;
 
-  _thunar_return_val_if_fail (THUNAR_IS_THUMBNAILER (thumbnailer), FALSE);
-  _thunar_return_val_if_fail (THUNAR_IS_FILE (file), FALSE);
-
-  /* fake a file list */
-  files.data = file;
-  files.next = NULL;
-  files.prev = NULL;
-
-  /* queue a thumbnail request for the file */
-  return thunar_thumbnailer_queue_files (thumbnailer, FALSE, &files, request, size);
-}
-
-
-
-gboolean
-thunar_thumbnailer_queue_files (ThunarThumbnailer   *thumbnailer,
-                                gboolean             lazy_checks,
-                                GList               *files,
-                                guint               *request,
-                                ThunarThumbnailSize  size)
-{
-  gboolean               success = FALSE;
-  ThunarThumbnailerJob  *job = NULL;
-
-  _thunar_return_val_if_fail (THUNAR_IS_THUMBNAILER (thumbnailer), FALSE);
-  _thunar_return_val_if_fail (files != NULL, FALSE);
+  _thunar_return_val_if_fail (THUNAR_IS_THUMBNAILER (thumbnailer), G_SOURCE_REMOVE);
 
   /* acquire the thumbnailer lock */
   _thumbnailer_lock (thumbnailer);
 
-  /* allocate a job */
-  job = g_slice_new0 (ThunarThumbnailerJob);
-  job->thumbnailer = thumbnailer;
-  job->files = g_list_copy_deep (files, (GCopyFunc) (void (*)(void)) g_object_ref, NULL);
-  job->lazy_checks = lazy_checks ? 1 : 0;
-  job->thumbnail_size = size;
-
   success = thunar_thumbnailer_begin_job (thumbnailer, job);
   if (success)
     {
       thumbnailer->jobs = g_slist_prepend (thumbnailer->jobs, job);
-      *request = job->request;
     }
   else
     {
+      for (GList* lp = job->files; lp != NULL; lp = lp->next)
+        {
+          /* This job failed .. inform all files which are waiting for the result */
+          thunar_file_update_thumbnail (lp->data, THUNAR_FILE_THUMB_STATE_NONE, job->thumbnail_size);
+        }
+
+      /* tell everybody we're done for that job */
+      g_signal_emit (G_OBJECT (thumbnailer), thumbnailer_signals[REQUEST_FINISHED], 0, job->request);
+
+      /* and drop it */
       thunar_thumbnailer_free_job (job);
     }
 
+  thumbnailer->jobs_to_queue[thumbnail_size] = NULL;
+  thumbnailer->jobs_to_queue_source_id[thumbnail_size] = 0;
+
   /* release the lock */
   _thumbnailer_unlock (thumbnailer);
 
-  return success;
+  return G_SOURCE_REMOVE;
+}
+
+
+
+void
+thunar_thumbnailer_queue_file (ThunarThumbnailer  *thumbnailer,
+                               ThunarFile         *file,
+                               guint              *request,
+                               ThunarThumbnailSize size)
+{
+  gint request_no;
+
+  _thunar_return_if_fail (THUNAR_IS_THUMBNAILER (thumbnailer));
+  _thunar_return_if_fail (THUNAR_IS_FILE (file));
+
+  if (thumbnailer->jobs_to_queue_source_id[size] == 0 )
+    {
+      ThunarThumbnailerJob  *job;
+
+      /* allocate a job */
+      job = g_slice_new0 (ThunarThumbnailerJob);
+      job->thumbnailer = thumbnailer;
+      job->files =  g_list_append (job->files, g_object_ref (file));
+      job->thumbnail_size = size;
+
+      /* queue a thumbnail request for the URIs from the wait queue */
+      /* compute the next request ID, making sure it's never 0 */
+      request_no = thumbnailer->last_request + 1;
+      request_no = MAX (request_no, 1);
+
+      /* remember the ID for the next request */
+      thumbnailer->last_request = request_no;
+
+      /* save the request number */
+      job->request = request_no;
+
+      thumbnailer->jobs_to_queue[size] = job;
+      thumbnailer->jobs_to_queue_source_id[size] = g_timeout_add (100, thunar_thumbnailer_queue_job_after_timeout, job);
+    }
+  else
+    {
+      if (thumbnailer->jobs_to_queue[size]->files == NULL)
+        g_warn_if_reached();
+
+      thumbnailer->jobs_to_queue[size]->files =  g_list_prepend (thumbnailer->jobs_to_queue[size]->files, g_object_ref (file));
+    }
+
+  *request = thumbnailer->jobs_to_queue[size]->request;
+  return;
 }
 
 
