@@ -1047,7 +1047,7 @@ thunar_tree_view_model_get_path (GtkTreeModel *model,
   _thunar_return_val_if_fail (iter->stamp == THUNAR_TREE_VIEW_MODEL (model)->stamp, NULL);
   _thunar_return_val_if_fail (iter->user_data != NULL, NULL);
 
-  g_assert (!g_sequence_iter_is_end (iter->user_data));
+  THUNAR_WARN_RETURN_VAL (g_sequence_iter_is_end (iter->user_data), NULL);
 
   node = g_sequence_get (iter->user_data);
 
@@ -1386,7 +1386,8 @@ thunar_tree_view_model_iter_next (GtkTreeModel *model,
   _thunar_return_val_if_fail (iter->stamp == THUNAR_TREE_VIEW_MODEL (model)->stamp, FALSE);
   _thunar_return_val_if_fail (iter->user_data != NULL, FALSE);
 
-  g_assert (!g_sequence_iter_is_end (iter->user_data));
+  THUNAR_WARN_RETURN_VAL (g_sequence_iter_is_end (iter->user_data), FALSE);
+
   ptr = g_sequence_iter_next (iter->user_data);
   if (g_sequence_iter_is_end (ptr))
     return FALSE;
@@ -1406,7 +1407,8 @@ thunar_tree_view_model_iter_previous (GtkTreeModel *model,
   _thunar_return_val_if_fail (iter->stamp == THUNAR_TREE_VIEW_MODEL (model)->stamp, FALSE);
   _thunar_return_val_if_fail (iter->user_data != NULL, FALSE);
 
-  g_assert (g_sequence_iter_is_begin (iter->user_data));
+  THUNAR_WARN_RETURN_VAL (g_sequence_iter_is_begin (iter->user_data), FALSE);
+
   ptr = g_sequence_iter_prev (iter->user_data);
   if (g_sequence_iter_is_begin (ptr))
     return FALSE;
@@ -1436,7 +1438,7 @@ thunar_tree_view_model_iter_children (GtkTreeModel *model,
     return FALSE;
 
   ptr = g_sequence_get_iter_at_pos (node->children, 0);
-  g_assert (!g_sequence_iter_is_end (ptr));
+  THUNAR_WARN_RETURN_VAL (g_sequence_iter_is_end (ptr), FALSE);
   GTK_TREE_ITER_INIT (*iter, THUNAR_TREE_VIEW_MODEL (model)->stamp, ptr);
 
   return TRUE;
@@ -1504,7 +1506,7 @@ thunar_tree_view_model_iter_nth_child (GtkTreeModel *model,
     return FALSE;
 
   ptr = g_sequence_get_iter_at_pos (node->children, n);
-  g_assert (!g_sequence_iter_is_end (ptr));
+  THUNAR_WARN_RETURN_VAL (g_sequence_iter_is_end (ptr), FALSE);
   GTK_TREE_ITER_INIT (*iter, THUNAR_TREE_VIEW_MODEL (model)->stamp, ptr);
 
   return TRUE;
@@ -1628,7 +1630,7 @@ _thunar_tree_view_model_search_finished (ThunarJob           *job,
 {
   if (model->search_job)
     {
-      g_signal_handlers_disconnect_matched (model->search_job, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, model);
+      g_signal_handlers_disconnect_by_data (model->search_job, model);
       g_object_unref (model->search_job);
       model->search_job = NULL;
     }
@@ -1656,7 +1658,7 @@ _thunar_tree_view_model_cancel_search_job (ThunarTreeViewModel *model)
     {
       exo_job_cancel (EXO_JOB (model->search_job));
 
-      g_signal_handlers_disconnect_matched (model->search_job, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, model);
+      g_signal_handlers_disconnect_by_data (model->search_job, model);
       g_object_unref (model->search_job);
       model->search_job = NULL;
     }
@@ -2030,7 +2032,8 @@ thunar_tree_view_model_new_node (ThunarFile *file)
 {
   Node *_node = g_malloc (sizeof (Node));
 
-  g_assert (file != NULL);
+  THUNAR_WARN_RETURN_VAL (file == NULL, NULL);
+  
   _node->file = g_object_ref (file);
 
   _node->dir = NULL;
@@ -2108,7 +2111,7 @@ thunar_tree_view_model_node_add_child (Node *node,
   GSequenceIter *iter;
   Node          *_child;
 
-  g_assert (child->file != NULL);
+  THUNAR_WARN_VOID_RETURN (child->file == NULL);
 
   child->depth = node->depth + 1;
   child->parent = node;
@@ -2118,11 +2121,14 @@ thunar_tree_view_model_node_add_child (Node *node,
     {
       /* replace the dummy node with the new child node */
       iter = g_sequence_get_iter_at_pos (node->children, 0);
-      g_assert (iter != NULL);
+
+      THUNAR_WARN_VOID_RETURN (iter == NULL);
+
       _child = g_sequence_get (iter);
       g_sequence_set (iter, child);
       child->ptr = iter;
-      g_assert (iter != NULL);
+
+      THUNAR_WARN_VOID_RETURN (iter == NULL);
 
       g_free (_child); /* free the dummy node */
 
@@ -2182,11 +2188,15 @@ thunar_tree_view_model_node_drop_dummy_child (Node *node)
 
   _thunar_return_if_fail (thunar_tree_view_model_node_has_dummy_child (node));
 
-  g_assert (node->n_children == 1);
+
+  THUNAR_WARN_VOID_RETURN (node->n_children != 1);
+
   iter = g_sequence_get_iter_at_pos (node->children, 0);
-  g_assert (!g_sequence_iter_is_end (iter));
+
+  THUNAR_WARN_VOID_RETURN (g_sequence_iter_is_end (iter));
+
   dummy = g_sequence_get (iter);
-  g_assert (iter != NULL && dummy->file == NULL);
+  THUNAR_WARN_VOID_RETURN (iter == NULL || dummy->file != NULL);
 
   GTK_TREE_ITER_INIT (tree_iter, node->model->stamp, dummy->ptr);
   path = gtk_tree_model_get_path (GTK_TREE_MODEL (node->model), &tree_iter);
@@ -2194,7 +2204,8 @@ thunar_tree_view_model_node_drop_dummy_child (Node *node)
   gtk_tree_path_free (path);
 
   node->n_children--;
-  g_assert (node->n_children == 0);
+
+  THUNAR_WARN_VOID_RETURN (node->n_children != 0);
 
   g_sequence_remove (dummy->ptr);
   g_free (dummy);
@@ -2218,7 +2229,7 @@ thunar_tree_view_model_dir_add_file (Node       *node,
   GtkTreePath *path;
   Node        *child;
 
-  g_assert (!g_hash_table_contains (node->set, file));
+  THUNAR_WARN_VOID_RETURN (g_hash_table_contains (node->set, file));
 
   child = thunar_tree_view_model_new_node (file);
   thunar_tree_view_model_node_add_child (node, child);
@@ -2248,11 +2259,8 @@ thunar_tree_view_model_dir_remove_file (Node       *node,
   GSequenceIter *iter;
 
   iter = g_hash_table_lookup (node->set, file);
-  if (iter == NULL)
-    {
-      g_warn_if_reached ();
-      return;
-    }
+
+  THUNAR_WARN_VOID_RETURN (iter == NULL);
 
   GTK_TREE_ITER_INIT (tree_iter, node->model->stamp, iter);
   path = gtk_tree_model_get_path (GTK_TREE_MODEL (node->model), &tree_iter);
@@ -2268,7 +2276,7 @@ thunar_tree_view_model_dir_remove_file (Node       *node,
   gtk_tree_model_row_deleted (GTK_TREE_MODEL (node->model), path);
   gtk_tree_path_free (path);
 
-  g_assert (node->n_children >= 0);
+  THUNAR_WARN_VOID_RETURN (node->n_children < 0);
 
   /* notify the model if all children have been deleted */
   if (node->ptr != NULL && node->n_children == 0)
@@ -2298,23 +2306,51 @@ thunar_tree_view_model_locate_file (ThunarTreeViewModel *model,
 
   parent = thunar_file_get_parent (file, NULL);
 
-  if (!g_hash_table_contains (model->subdirs, parent))
+  if (parent == NULL || !g_hash_table_contains (model->subdirs, parent))
     return NULL;
 
   parent_node = g_hash_table_lookup (model->subdirs, parent);
-  g_assert (parent_node != NULL);
+  g_object_unref (parent);
+  THUNAR_WARN_RETURN_VAL (parent_node == NULL, NULL);
 
   if (!g_hash_table_contains (parent_node->set, file))
     return NULL;
 
   iter = g_hash_table_lookup (parent_node->set, file);
-  g_assert (iter != NULL);
+  THUNAR_WARN_RETURN_VAL (iter == NULL, NULL);
 
   return g_sequence_get (iter);
 }
 
 
 
+static Node *
+thunar_tree_view_model_locate_parent (ThunarTreeViewModel *model,
+                                      ThunarFile          *file)
+{
+  ThunarFile    *parent;
+  Node          *parent_node;
+
+  _thunar_return_val_if_fail (THUNAR_IS_TREE_VIEW_MODEL (model), NULL);
+  _thunar_return_val_if_fail (THUNAR_IS_FILE (file), NULL);
+
+  if (!thunar_file_has_parent (file))
+    return NULL;
+
+  parent = thunar_file_get_parent (file, NULL);
+
+  if (parent == NULL || !g_hash_table_contains (model->subdirs, parent))
+    return NULL;
+
+  parent_node = g_hash_table_lookup (model->subdirs, parent);
+  g_object_unref (parent);
+  THUNAR_WARN_RETURN_VAL (parent_node == NULL, NULL);
+
+  return parent_node;
+}
+
+
+
 static gint
 thunar_tree_view_model_cmp_nodes (gconstpointer a,
                                   gconstpointer b,
@@ -2449,18 +2485,21 @@ _thunar_tree_view_model_folder_error (Node         *node,
 {
   _thunar_return_if_fail (error != NULL);
 
-  if (node->parent == NULL && node->n_children > 0)
+  if (node->n_children > 0)
     {
-      /* set_folder func does not emit row-deleted signal instead relies
-       * on ThunarStandardView to disconnect & reconnect the view quickly update
-       * changes in current_directory. */
-      for (GList *lp = g_hash_table_get_keys (node->set); lp != NULL; lp = lp->next)
-        thunar_tree_view_model_dir_remove_file (node, THUNAR_FILE (lp->data));
-      /* reset the model if error is with the current directory */
-      thunar_tree_view_model_set_folder (THUNAR_STANDARD_VIEW_MODEL (node->model), NULL, NULL);
+      if (node->parent == NULL)
+        {
+          /* set_folder func does not emit row-deleted signal instead relies
+          * on ThunarStandardView to disconnect & reconnect the view quickly update
+          * changes in current_directory. */
+          for (GList *lp = g_hash_table_get_keys (node->set); lp != NULL; lp = lp->next)
+            thunar_tree_view_model_dir_remove_file (node, THUNAR_FILE (lp->data));
+          /* reset the model if error is with the current directory */
+          thunar_tree_view_model_set_folder (THUNAR_STANDARD_VIEW_MODEL (node->model), NULL, NULL);
+        }
+      else
+        thunar_tree_view_model_dir_remove_file (node->parent, node->file);
     }
-  else
-    thunar_tree_view_model_dir_remove_file (node->parent, node->file);
 
   /* forward the error signal */
   g_signal_emit (G_OBJECT (node->model), tree_model_signals[THUNAR_STANDARD_VIEW_MODEL_ERROR], 0, error);
@@ -2479,7 +2518,7 @@ _thunar_tree_view_model_dir_files_added (Node  *node,
     {
       file = THUNAR_FILE (lp->data);
 
-      if (thunar_file_is_hidden (file))
+      if (thunar_file_is_hidden (file) && !g_hash_table_contains (node->hidden_files, file))
         {
           g_hash_table_add (node->hidden_files, g_object_ref (file));
 
@@ -2555,7 +2594,7 @@ thunar_tree_view_model_load_dir (Node *node)
   thunar_tree_view_model_set_loading (node->model, TRUE);
 
   node->dir = thunar_folder_get_for_file (node->file);
-  g_assert (node->dir != NULL);
+  THUNAR_WARN_VOID_RETURN (node->dir == NULL);
 
   g_signal_connect_swapped (G_OBJECT (node->dir), "destroy", G_CALLBACK (_thunar_tree_view_model_folder_destroy), node);
   g_signal_connect_swapped (G_OBJECT (node->dir), "error", G_CALLBACK (_thunar_tree_view_model_folder_error), node);
@@ -2607,9 +2646,10 @@ thunar_tree_view_model_cleanup_model (ThunarTreeViewModel *model)
   for (GList *dirs = values; dirs != NULL; dirs = dirs->next)
     {
       _node = dirs->data;
-      g_assert (_node != NULL && _node->dir != NULL);
-      g_signal_handlers_disconnect_matched (G_OBJECT (_node->dir), G_SIGNAL_MATCH_DATA,
-                                            0, 0, NULL, NULL, _node);
+
+      THUNAR_WARN_VOID_RETURN (_node == NULL || _node->dir == NULL);
+
+      g_signal_handlers_disconnect_by_data (G_OBJECT (_node->dir), _node);
       g_object_unref (_node->dir);
       _node->dir = NULL;
     }
@@ -2661,12 +2701,14 @@ thunar_tree_view_model_node_destroy (Node *node)
       return;
     }
 
-  g_assert (node->file != NULL);
+  THUNAR_WARN_VOID_RETURN (node->file == NULL);
 
   if (node->dir != NULL)
     {
       g_signal_handlers_disconnect_by_data (G_OBJECT (node->dir), node);
-      g_assert (g_hash_table_contains (node->model->subdirs, node->file));
+
+      THUNAR_WARN_VOID_RETURN (!g_hash_table_contains (node->model->subdirs, node->file));
+
       g_hash_table_remove (node->model->subdirs, node->file);
       g_object_unref (node->dir);
     }
@@ -2697,7 +2739,7 @@ thunar_tree_view_model_file_changed (ThunarFileMonitor   *monitor,
   GSequenceIter *iter;
   GtkTreeIter    tree_iter;
   GtkTreePath   *path;
-  Node          *node;
+  Node          *node, *parent;
   gint           pos_after, pos_before = 0;
   gint          *new_order;
   gint           length;
@@ -2706,7 +2748,33 @@ thunar_tree_view_model_file_changed (ThunarFileMonitor   *monitor,
   node = thunar_tree_view_model_locate_file (model, file);
 
   if (node == NULL)
-    return;
+    {
+      parent = thunar_tree_view_model_locate_parent (model, file);
+
+      if (parent == NULL)
+        return;
+
+      /* two cases - file is hidden or not
+       * 1. if it is in hidden list but not hidden anymore then add the new file
+       * 2. if it is not hidden but has turned hidden hide it */
+      if (g_hash_table_contains (parent->hidden_files, file) && !thunar_file_is_hidden (file))
+        {
+          g_hash_table_remove (parent->hidden_files, file);
+          thunar_tree_view_model_dir_add_file (parent, file);
+          return;
+        }
+
+      /* file is now hidden but still in the visible list */
+      if (g_hash_table_contains (parent->set, file) && thunar_file_is_hidden (file))
+        {
+          if (!model->show_hidden)
+            thunar_tree_view_model_dir_remove_file (parent, file);
+          g_hash_table_add (parent->hidden_files, g_object_ref (file));
+          return;
+        }
+
+      return;
+    }
 
   if (thunar_file_is_hidden (file))
     {
@@ -2809,9 +2877,11 @@ thunar_tree_view_model_load_subdir (ThunarTreeViewModel *model,
   _thunar_return_if_fail (iter->stamp == model->stamp);
   _thunar_return_if_fail (iter->user_data != NULL);
 
-  g_assert (!g_sequence_iter_is_end (iter->user_data));
+  THUNAR_WARN_VOID_RETURN (g_sequence_iter_is_end (iter->user_data));
+
   node = g_sequence_get (iter->user_data);
-  g_assert (node != NULL);
+  THUNAR_WARN_VOID_RETURN (node == NULL);
+
   node->expanded = TRUE;
   if (node->scheduled_unload_id != 0)
     g_source_remove (node->scheduled_unload_id);
@@ -2833,8 +2903,7 @@ _thunar_tree_view_model_dir_unload_timeout (Node *node)
 
   if (node->dir != NULL)
     {
-      g_signal_handlers_disconnect_matched (G_OBJECT (node->dir), G_SIGNAL_MATCH_DATA,
-                                            0, 0, NULL, NULL, node);
+      g_signal_handlers_disconnect_by_data (G_OBJECT (node->dir), node);
       g_object_unref (node->dir);
       node->dir = NULL;
     }
@@ -2885,9 +2954,11 @@ thunar_tree_view_model_schedule_unload (ThunarTreeViewModel *model,
   _thunar_return_if_fail (iter->stamp == model->stamp);
   _thunar_return_if_fail (iter->user_data != NULL);
 
-  g_assert (!g_sequence_iter_is_end (iter->user_data));
+  THUNAR_WARN_VOID_RETURN (g_sequence_iter_is_end (iter->user_data));
+
   node = g_sequence_get (iter->user_data);
-  g_assert (node != NULL);
+  THUNAR_WARN_VOID_RETURN (node == NULL);
+
   node->expanded = FALSE;
   node->scheduled_unload_id = g_timeout_add_full (G_PRIORITY_LOW, CLEANUP_AFTER_COLLAPSE_DELAY,
                                                   G_SOURCE_FUNC (_thunar_tree_view_model_dir_unload_timeout),
@@ -2925,7 +2996,6 @@ thunar_tree_view_model_update_search_files (ThunarTreeViewModel *model)
   if (model->search_files != NULL)
     g_list_free (model->search_files);
   model->search_files = NULL;
-
   g_mutex_unlock (&model->mutex_add_search_files);
 
   return TRUE;
