@@ -20,7 +20,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 #ifdef HAVE_TIME_H
 #include <time.h>
@@ -30,24 +30,24 @@
 
 #include <libxfce4util/libxfce4util.h>
 
-#include <thunar/thunar-application.h>
-#include <thunar/thunar-clipboard-manager.h>
-#include <thunar/thunar-device.h>
-#include <thunar/thunar-dialogs.h>
-#include <thunar/thunar-dnd.h>
-#include <thunar/thunar-gio-extensions.h>
-#include <thunar/thunar-gtk-extensions.h>
-#include <thunar/thunar-job.h>
-#include <thunar/thunar-marshal.h>
-#include <thunar/thunar-menu.h>
-#include <thunar/thunar-preferences.h>
-#include <thunar/thunar-private.h>
-#include <thunar/thunar-properties-dialog.h>
-#include <thunar/thunar-shortcuts-icon-renderer.h>
-#include <thunar/thunar-simple-job.h>
-#include <thunar/thunar-tree-model.h>
-#include <thunar/thunar-tree-view.h>
-#include <thunar/thunar-util.h>
+#include "thunar/thunar-application.h"
+#include "thunar/thunar-clipboard-manager.h"
+#include "thunar/thunar-device.h"
+#include "thunar/thunar-dialogs.h"
+#include "thunar/thunar-dnd.h"
+#include "thunar/thunar-gio-extensions.h"
+#include "thunar/thunar-gtk-extensions.h"
+#include "thunar/thunar-job.h"
+#include "thunar/thunar-marshal.h"
+#include "thunar/thunar-menu.h"
+#include "thunar/thunar-preferences.h"
+#include "thunar/thunar-private.h"
+#include "thunar/thunar-properties-dialog.h"
+#include "thunar/thunar-shortcuts-icon-renderer.h"
+#include "thunar/thunar-simple-job.h"
+#include "thunar/thunar-tree-model.h"
+#include "thunar/thunar-tree-view.h"
+#include "thunar/thunar-util.h"
 
 
 
@@ -1453,6 +1453,7 @@ thunar_tree_view_action_unlink_selected_folder (ThunarTreeView *view,
   ThunarApplication *application;
   ThunarFile        *file;
   GList              file_list;
+  gboolean           warn;
 
   _thunar_return_if_fail (THUNAR_IS_TREE_VIEW (view));
 
@@ -1468,8 +1469,12 @@ thunar_tree_view_action_unlink_selected_folder (ThunarTreeView *view,
           file_list.prev = NULL;
 
           /* delete the file */
+          if (permanently)
+            warn = TRUE;
+          else
+            g_object_get (G_OBJECT (view->preferences), "misc-confirm-move-to-trash", &warn, NULL);
           application = thunar_application_get ();
-          thunar_application_unlink_files (application, GTK_WIDGET (view), &file_list, permanently, TRUE);
+          thunar_application_unlink_files (application, GTK_WIDGET (view), &file_list, permanently, warn, THUNAR_OPERATION_LOG_OPERATIONS);
           g_object_unref (G_OBJECT (application));
         }
 
@@ -1668,8 +1673,6 @@ thunar_tree_view_set_cursor (gpointer user_data)
       return FALSE;
     }
 
-THUNAR_THREADS_ENTER
-
   /* for easier navigation, we sometimes want to force/keep selection of a certain path */
   if (view->select_path != NULL)
     {
@@ -1792,8 +1795,6 @@ THUNAR_THREADS_ENTER
     }
   g_list_free (path_as_list);
 
-THUNAR_THREADS_LEAVE
-
   return !done;
 }
 
@@ -1822,8 +1823,6 @@ thunar_tree_view_drag_scroll_timer (gpointer user_data)
   gint            offset;
   gint            y, h;
 
-THUNAR_THREADS_ENTER
-
   /* verify that we are realized */
   if (gtk_widget_get_realized (GTK_WIDGET (view)))
     {
@@ -1893,8 +1892,6 @@ THUNAR_THREADS_ENTER
         }
     }
 
-THUNAR_THREADS_LEAVE
-
   return TRUE;
 }
 
@@ -1914,8 +1911,6 @@ thunar_tree_view_expand_timer (gpointer user_data)
   ThunarTreeView *view = THUNAR_TREE_VIEW (user_data);
   GtkTreePath    *path;
 
-THUNAR_THREADS_ENTER
-
   /* cancel the drag autoscroll timer when expanding a row */
   if (G_UNLIKELY (view->drag_scroll_timer_id != 0))
     g_source_remove (view->drag_scroll_timer_id);
@@ -1929,8 +1924,6 @@ THUNAR_THREADS_ENTER
       gtk_tree_path_free (path);
     }
 
-THUNAR_THREADS_LEAVE
-
   return FALSE;
 }
 
