@@ -20,7 +20,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #ifdef HAVE_SYS_TYPES_H
@@ -54,11 +54,14 @@
 #include <glib/gwin32.h>
 #endif
 
-#include <thunar/thunar-private.h>
-#include <thunar/thunar-util.h>
-#include <thunar/thunar-folder.h>
-#include <thunar/thunar-text-renderer.h>
-#include <thunar/thunar-icon-renderer.h>
+#include "thunar/thunar-private.h"
+#include "thunar/thunar-util.h"
+#include "thunar/thunar-folder.h"
+#include "thunar/thunar-text-renderer.h"
+#include "thunar/thunar-icon-renderer.h"
+#include "thunar/thunar-preferences.h"
+#include "thunar/thunar-renamer-dialog.h"
+#include "thunar/thunar-window.h"
 
 #include <glib.h>
 #include <glib/gstdio.h>
@@ -82,8 +85,6 @@ enum
   DRAW_ON_ALL_SIDES,
 } DrawOnSide;
 
-const char *SEARCH_PREFIX = "Search: ";
-
 /**
  * thunar_util_strrchr_offset:
  * @str:    haystack
@@ -690,6 +691,7 @@ thunar_setup_display_cb (gpointer data)
  * @dir : the directory to search for a free filename
  * @file_name : the filename which will be used as the basis/default
  * @ThunarNextFileNameMode: To decide if the naming should follow "file copy","file link" or "new file" syntax
+ * @is_directory: TRUE, if @file_name is supposed to be the name of a directory
  *
  * Returns a filename that is like @file_name with the possible addition of
  * a number to differentiate it from other similarly named files. In other words
@@ -711,7 +713,8 @@ thunar_setup_display_cb (gpointer data)
 gchar*
 thunar_util_next_new_file_name (ThunarFile            *dir,
                                 const gchar           *file_name,
-                                ThunarNextFileNameMode name_mode)
+                                ThunarNextFileNameMode name_mode,
+                                gboolean               is_directory)
 {
   ThunarFolder   *folder          = thunar_folder_get_for_file (dir);
   unsigned long   file_name_size  = strlen (file_name);
@@ -721,7 +724,8 @@ thunar_util_next_new_file_name (ThunarFile            *dir,
   gchar          *new_name        = g_strdup (file_name);
 
   /* get file extension if file is not a directory */
-  extension = thunar_util_str_get_extension (file_name);
+  if (!is_directory)
+    extension = thunar_util_str_get_extension (file_name);
 
   /* if the file has an extension don't include it in the search */
   if (extension != NULL)
@@ -730,8 +734,9 @@ thunar_util_next_new_file_name (ThunarFile            *dir,
   /* loop through the directory until new_name is unique */
   while (TRUE)
     {
+      GList *files = thunar_folder_get_files (folder);
       found_duplicate = FALSE;
-      for (GList *files = thunar_folder_get_files (folder); files != NULL; files = files->next)
+      for (;files != NULL; files = files->next)
         {
           ThunarFile  *file = files->data;
           gchar       *name = g_file_get_basename (thunar_file_get_file (file));
@@ -744,6 +749,7 @@ thunar_util_next_new_file_name (ThunarFile            *dir,
             }
           g_free (name);
         }
+      g_list_free (files);
 
       if (!found_duplicate)
         break;
@@ -774,6 +780,19 @@ thunar_util_next_new_file_name (ThunarFile            *dir,
 
 
 
+/**
+ * thunar_util_get_search_prefix
+ *
+ * Return value: The localized search prefix
+**/
+const char*
+thunar_util_get_search_prefix (void)
+{
+  return _("Search: ");
+}
+
+
+
 /**
  * thunar_util_is_a_search_query
  * @string : the string to check
@@ -783,7 +802,7 @@ thunar_util_next_new_file_name (ThunarFile            *dir,
 gboolean
 thunar_util_is_a_search_query (const gchar *string)
 {
-  return strncmp (string, SEARCH_PREFIX, strlen (SEARCH_PREFIX)) == 0;
+  return strncmp (string, thunar_util_get_search_prefix (), strlen (thunar_util_get_search_prefix ())) == 0;
 }
 
 
@@ -953,6 +972,8 @@ thunar_util_clip_view_background (GtkCellRenderer      *cell,
   gchar            *highlight_color;
   gdouble           radius = 0.0;
   gint              side = DRAW_ON_ALL_SIDES;
+  GtkWidget        *toplevel;
+  gboolean          window_is_backdrop = TRUE;
 
   g_object_get (G_OBJECT (cell), 
                 "highlight-color", &highlight_color,
@@ -977,6 +998,11 @@ thunar_util_clip_view_background (GtkCellRenderer      *cell,
       color = gdk_rgba_copy (&highlight_color_rgba);
     }
 
+  /* check the state of our window (active/backdrop) */
+  toplevel = gtk_widget_get_toplevel (widget);
+  if (GTK_IS_WINDOW (toplevel) && gtk_window_is_active (GTK_WINDOW (toplevel)))
+    window_is_backdrop = FALSE;
+
   /**
    * If the item is selected then paint the background area with the theme's selected item's color.
    * To distinguish between highlighted & non highlighted files, the background area of icon renderer
@@ -985,16 +1011,127 @@ thunar_util_clip_view_background (GtkCellRenderer      *cell,
   if (G_UNLIKELY (color_selected && !(THUNAR_IS_ICON_RENDERER (cell) && highlight_color != NULL)))
     {
       context = gtk_widget_get_style_context (widget);
-      gtk_style_context_get (context, GTK_STATE_FLAG_SELECTED, GTK_STYLE_PROPERTY_BACKGROUND_COLOR, &color, NULL);
+      if (window_is_backdrop)
+        gtk_style_context_lookup_color (context, "theme_unfocused_selected_bg_color", &highlight_color_rgba);
+      else
+        gtk_style_context_lookup_color (context, "theme_selected_bg_color", &highlight_color_rgba);
+      color = gdk_rgba_copy (&highlight_color_rgba);
     }
 
   if (G_LIKELY (color != NULL))
     {
       gdk_cairo_set_source_rgba (cr, color);
       gdk_rgba_free (color);
-      cairo_paint (cr);
+      cairo_paint_with_alpha (cr, window_is_backdrop ? ALPHA_BACKDROP : ALPHA_FOCUSED);
     }
 
   g_free (highlight_color);
   cairo_restore (cr);
 }
+
+
+
+/**
+ * thunar_util_split_search_query:
+ * @search_query: The search query to split.
+ * @error: Return location for regex compilation errors.
+ *
+ * Search terms are split on whitespace. Search queries must be
+ * normalized before passing to this function.
+ *
+ * See also: thunar_g_utf8_normalize_for_search().
+ *
+ * Return value: a list of search terms which must be freed with g_strfreev()
+ **/
+gchar **
+thunar_util_split_search_query (const gchar *search_query_normalized,
+                                GError     **error)
+{
+  GRegex *whitespace_regex;
+  gchar **search_terms;
+
+  whitespace_regex = g_regex_new ("\\s+", 0, 0, error);
+  if (whitespace_regex == NULL)
+    return NULL;
+  search_terms = g_regex_split (whitespace_regex, search_query_normalized, 0);
+  g_regex_unref (whitespace_regex);
+  return search_terms;
+}
+
+
+
+/**
+ * thunar_util_search_terms_match:
+ * @terms: The search terms to look for, prepared with thunar_util_split_search_query().
+ * @str: The string which the search terms might be found in.
+ *
+ * All search terms must match. Thunar uses simple substring matching
+ * for the broadest multilingual support. @str must be normalized before
+ * passing to this function.
+ *
+ * See also: thunar_g_utf8_normalize_for_search().
+ *
+ * Return value: TRUE if all terms matched, FALSE otherwise.
+ **/
+
+gboolean
+thunar_util_search_terms_match (gchar **terms,
+                                gchar  *str)
+{
+  for (gint i = 0; terms[i] != NULL; i++)
+    if (g_strrstr (str, terms[i]) == NULL)
+      return FALSE;
+  return TRUE;
+}
+
+
+
+gboolean
+thunar_util_save_geometry_timer (gpointer user_data)
+{
+  GdkWindowState         state;
+  ThunarPreferences     *preferences;
+  gboolean               remember_geometry;
+  gint                   width;
+  gint                   height;
+  
+  preferences = thunar_preferences_get ();
+  g_object_get (G_OBJECT (preferences), "misc-remember-geometry", &remember_geometry, NULL);
+  
+  /* check if we should remember the window geometry */
+  if (G_LIKELY (remember_geometry))
+    {
+      /* check if the window is still visible */
+      if (gtk_widget_get_visible (user_data))
+        {
+          /* determine the current state of the window */
+          state = gdk_window_get_state (gtk_widget_get_window (user_data));
+
+          /* don't save geometry for maximized or fullscreen windows */
+          if ((state & (GDK_WINDOW_STATE_MAXIMIZED | GDK_WINDOW_STATE_FULLSCREEN)) == 0)
+            {
+              /* determine the current width/height of the window... */
+              gtk_window_get_size (GTK_WINDOW (user_data), &width, &height);
+
+              /* ...and remember them as default for new windows */
+              if (THUNAR_IS_WINDOW (user_data))
+                g_object_set (G_OBJECT (preferences), "last-window-width", width, "last-window-height", height,
+                              "last-window-maximized", FALSE, NULL);
+              else if (THUNAR_IS_RENAMER_DIALOG (user_data))
+                g_object_set (G_OBJECT (preferences), "last-renamer-dialog-width", width, "last-renamer-dialog-height", height,
+                              "last-renamer-dialog-maximized", FALSE, NULL);
+            }
+          else
+            {
+              /* only store that the window is full screen */
+              if (THUNAR_IS_WINDOW (user_data))
+                g_object_set (G_OBJECT (preferences), "last-window-maximized", TRUE, NULL);
+              else if (THUNAR_IS_RENAMER_DIALOG (user_data))
+                g_object_set (G_OBJECT (preferences), "last-renamer-dialog-maximized", TRUE, NULL);
+            }
+        }
+    }
+    g_object_unref(preferences);
+
+  return G_SOURCE_REMOVE;
+}
\ No newline at end of file
