@@ -19,11 +19,13 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
-#include <thunarx/thunarx-private.h>
-#include <thunarx/thunarx-property-page-provider.h>
+#include <libxfce4util/libxfce4util.h>
+
+#include "thunarx/thunarx-private.h"
+#include "thunarx/thunarx-property-page-provider.h"
 
 /**
  * SECTION: thunarx-property-page-provider
